/********************************************************************************
** Form generated from reading UI file 'io_report.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_IO_REPORT_H
#define UI_IO_REPORT_H

#include <QtCore/QVariant>
#include <QtGui/QIcon>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QFrame>
#include <QtWidgets/QLabel>
#include <QtWidgets/QListWidget>
#include <QtWidgets/QVBoxLayout>

QT_BEGIN_NAMESPACE

class Ui_io_report
{
public:
    QVBoxLayout *verticalLayout;
    QLabel *label_2;
    QListWidget *listInputErrors;
    QCheckBox *cbRemoveFiles;
    QFrame *line;
    QLabel *label;
    QListWidget *listOutputErrors;
    QFrame *line_2;
    QLabel *label_3;
    QListWidget *listReformatErrors;
    QFrame *line_3;
    QLabel *label_4;
    QListWidget *listSyntaxErrors;
    QDialogButtonBox *buttonBox;

    void setupUi(QDialog *io_report)
    {
        if (io_report->objectName().isEmpty())
            io_report->setObjectName("io_report");
        io_report->resize(442, 453);
        QIcon icon;
        icon.addFile(QString::fromUtf8(":/highlight.png"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        io_report->setWindowIcon(icon);
        verticalLayout = new QVBoxLayout(io_report);
        verticalLayout->setObjectName("verticalLayout");
        label_2 = new QLabel(io_report);
        label_2->setObjectName("label_2");

        verticalLayout->addWidget(label_2);

        listInputErrors = new QListWidget(io_report);
        listInputErrors->setObjectName("listInputErrors");

        verticalLayout->addWidget(listInputErrors);

        cbRemoveFiles = new QCheckBox(io_report);
        cbRemoveFiles->setObjectName("cbRemoveFiles");

        verticalLayout->addWidget(cbRemoveFiles);

        line = new QFrame(io_report);
        line->setObjectName("line");
        line->setFrameShape(QFrame::Shape::HLine);
        line->setFrameShadow(QFrame::Shadow::Sunken);

        verticalLayout->addWidget(line);

        label = new QLabel(io_report);
        label->setObjectName("label");

        verticalLayout->addWidget(label);

        listOutputErrors = new QListWidget(io_report);
        listOutputErrors->setObjectName("listOutputErrors");

        verticalLayout->addWidget(listOutputErrors);

        line_2 = new QFrame(io_report);
        line_2->setObjectName("line_2");
        line_2->setFrameShape(QFrame::Shape::HLine);
        line_2->setFrameShadow(QFrame::Shadow::Sunken);

        verticalLayout->addWidget(line_2);

        label_3 = new QLabel(io_report);
        label_3->setObjectName("label_3");

        verticalLayout->addWidget(label_3);

        listReformatErrors = new QListWidget(io_report);
        listReformatErrors->setObjectName("listReformatErrors");

        verticalLayout->addWidget(listReformatErrors);

        line_3 = new QFrame(io_report);
        line_3->setObjectName("line_3");
        line_3->setFrameShape(QFrame::Shape::HLine);
        line_3->setFrameShadow(QFrame::Shadow::Sunken);

        verticalLayout->addWidget(line_3);

        label_4 = new QLabel(io_report);
        label_4->setObjectName("label_4");

        verticalLayout->addWidget(label_4);

        listSyntaxErrors = new QListWidget(io_report);
        listSyntaxErrors->setObjectName("listSyntaxErrors");

        verticalLayout->addWidget(listSyntaxErrors);

        buttonBox = new QDialogButtonBox(io_report);
        buttonBox->setObjectName("buttonBox");
        buttonBox->setOrientation(Qt::Horizontal);
        buttonBox->setStandardButtons(QDialogButtonBox::Ok);

        verticalLayout->addWidget(buttonBox);


        retranslateUi(io_report);
        QObject::connect(buttonBox, &QDialogButtonBox::accepted, io_report, qOverload<>(&QDialog::accept));
        QObject::connect(buttonBox, &QDialogButtonBox::rejected, io_report, qOverload<>(&QDialog::reject));

        QMetaObject::connectSlotsByName(io_report);
    } // setupUi

    void retranslateUi(QDialog *io_report)
    {
        io_report->setWindowTitle(QCoreApplication::translate("io_report", "Error summary", nullptr));
        label_2->setText(QCoreApplication::translate("io_report", "Input errors:", nullptr));
        cbRemoveFiles->setText(QCoreApplication::translate("io_report", "Remove files above from input file list", nullptr));
        label->setText(QCoreApplication::translate("io_report", "Output errors:", nullptr));
        label_3->setText(QCoreApplication::translate("io_report", "Reformatting not possible:", nullptr));
        label_4->setText(QCoreApplication::translate("io_report", "Failed syntax tests:", nullptr));
    } // retranslateUi

};

namespace Ui {
    class io_report: public Ui_io_report {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_IO_REPORT_H
