/****************************************************************************
** Meta object code from reading C++ file 'mainwindow.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "mainwindow.h"
#include <QtGui/qtextcursor.h>
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'mainwindow.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN10MainWindowE_t {};
} // unnamed namespace

template <> constexpr inline auto MainWindow::qt_create_metaobjectdata<qt_meta_tag_ZN10MainWindowE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "MainWindow",
        "on_pbClearSelection_clicked",
        "",
        "on_pbOutputDest_clicked",
        "on_pbStartConversion_clicked",
        "on_pbCopyFile2CP_clicked",
        "on_actionVisit_website_triggered",
        "on_pbClearAllPlugins_clicked",
        "on_pbClearAll_clicked",
        "on_pbClearSelPlugin_clicked",
        "on_pbSelectPlugin_clicked",
        "on_pbSelectScript_clicked",
        "on_pbClearAllScripts_clicked",
        "on_pbClearSelScript_clicked",
        "on_pbCopyToCP_clicked",
        "on_pbPasteFromCB_clicked",
        "on_actionAbout_translations_triggered",
        "on_pbTEXChooseStyleIncFile_clicked",
        "on_pbLATEXChooseStyleIncFile_clicked",
        "on_pbHTMLChooseStyleIncFile_clicked",
        "on_pbSVGChooseStyleIncFile_clicked",
        "on_pbCopyAndPaste_clicked",
        "on_pbLSInitialize_clicked",
        "on_leLSExec_textChanged",
        "on_pbSelWorkspace_clicked",
        "on_pbSelExecutable_clicked",
        "on_action_License_triggered",
        "on_action_Changelog_triggered",
        "on_action_Plug_Ins_triggered",
        "on_action_Manual_triggered",
        "on_action_About_Highlight_triggered",
        "on_action_Exit_triggered",
        "plausibility",
        "updatePreview",
        "openFiles",
        "loadLSProfile",
        "on_pbPluginReadFilePath_clicked",
        "on_pbBrowseOutDir_clicked",
        "on_lvUserScripts_itemClicked",
        "QListWidgetItem*",
        "item",
        "on_lvPluginScripts_itemClicked",
        "on_browserPreview_selectionChanged",
        "on_comboThemeFilter_currentIndexChanged",
        "index"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'on_pbClearSelection_clicked'
        QtMocHelpers::SlotData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'on_pbOutputDest_clicked'
        QtMocHelpers::SlotData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'on_pbStartConversion_clicked'
        QtMocHelpers::SlotData<void()>(4, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'on_pbCopyFile2CP_clicked'
        QtMocHelpers::SlotData<void()>(5, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'on_actionVisit_website_triggered'
        QtMocHelpers::SlotData<void()>(6, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'on_pbClearAllPlugins_clicked'
        QtMocHelpers::SlotData<void()>(7, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'on_pbClearAll_clicked'
        QtMocHelpers::SlotData<void()>(8, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'on_pbClearSelPlugin_clicked'
        QtMocHelpers::SlotData<void()>(9, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'on_pbSelectPlugin_clicked'
        QtMocHelpers::SlotData<void()>(10, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'on_pbSelectScript_clicked'
        QtMocHelpers::SlotData<void()>(11, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'on_pbClearAllScripts_clicked'
        QtMocHelpers::SlotData<void()>(12, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'on_pbClearSelScript_clicked'
        QtMocHelpers::SlotData<void()>(13, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'on_pbCopyToCP_clicked'
        QtMocHelpers::SlotData<void()>(14, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'on_pbPasteFromCB_clicked'
        QtMocHelpers::SlotData<void()>(15, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'on_actionAbout_translations_triggered'
        QtMocHelpers::SlotData<void()>(16, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'on_pbTEXChooseStyleIncFile_clicked'
        QtMocHelpers::SlotData<void()>(17, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'on_pbLATEXChooseStyleIncFile_clicked'
        QtMocHelpers::SlotData<void()>(18, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'on_pbHTMLChooseStyleIncFile_clicked'
        QtMocHelpers::SlotData<void()>(19, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'on_pbSVGChooseStyleIncFile_clicked'
        QtMocHelpers::SlotData<void()>(20, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'on_pbCopyAndPaste_clicked'
        QtMocHelpers::SlotData<void()>(21, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'on_pbLSInitialize_clicked'
        QtMocHelpers::SlotData<void()>(22, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'on_leLSExec_textChanged'
        QtMocHelpers::SlotData<void()>(23, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'on_pbSelWorkspace_clicked'
        QtMocHelpers::SlotData<void()>(24, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'on_pbSelExecutable_clicked'
        QtMocHelpers::SlotData<void()>(25, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'on_action_License_triggered'
        QtMocHelpers::SlotData<void()>(26, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'on_action_Changelog_triggered'
        QtMocHelpers::SlotData<void()>(27, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'on_action_Plug_Ins_triggered'
        QtMocHelpers::SlotData<void()>(28, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'on_action_Manual_triggered'
        QtMocHelpers::SlotData<void()>(29, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'on_action_About_Highlight_triggered'
        QtMocHelpers::SlotData<void()>(30, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'on_action_Exit_triggered'
        QtMocHelpers::SlotData<void()>(31, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'plausibility'
        QtMocHelpers::SlotData<void()>(32, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'updatePreview'
        QtMocHelpers::SlotData<void()>(33, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'openFiles'
        QtMocHelpers::SlotData<void()>(34, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'loadLSProfile'
        QtMocHelpers::SlotData<void()>(35, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'on_pbPluginReadFilePath_clicked'
        QtMocHelpers::SlotData<void()>(36, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'on_pbBrowseOutDir_clicked'
        QtMocHelpers::SlotData<void()>(37, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'on_lvUserScripts_itemClicked'
        QtMocHelpers::SlotData<void(QListWidgetItem *)>(38, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 39, 40 },
        }}),
        // Slot 'on_lvPluginScripts_itemClicked'
        QtMocHelpers::SlotData<void(QListWidgetItem *)>(41, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 39, 40 },
        }}),
        // Slot 'on_browserPreview_selectionChanged'
        QtMocHelpers::SlotData<void()>(42, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'on_comboThemeFilter_currentIndexChanged'
        QtMocHelpers::SlotData<void(int)>(43, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 44 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<MainWindow, qt_meta_tag_ZN10MainWindowE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject MainWindow::staticMetaObject = { {
    QMetaObject::SuperData::link<QMainWindow::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10MainWindowE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10MainWindowE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10MainWindowE_t>.metaTypes,
    nullptr
} };

void MainWindow::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<MainWindow *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->on_pbClearSelection_clicked(); break;
        case 1: _t->on_pbOutputDest_clicked(); break;
        case 2: _t->on_pbStartConversion_clicked(); break;
        case 3: _t->on_pbCopyFile2CP_clicked(); break;
        case 4: _t->on_actionVisit_website_triggered(); break;
        case 5: _t->on_pbClearAllPlugins_clicked(); break;
        case 6: _t->on_pbClearAll_clicked(); break;
        case 7: _t->on_pbClearSelPlugin_clicked(); break;
        case 8: _t->on_pbSelectPlugin_clicked(); break;
        case 9: _t->on_pbSelectScript_clicked(); break;
        case 10: _t->on_pbClearAllScripts_clicked(); break;
        case 11: _t->on_pbClearSelScript_clicked(); break;
        case 12: _t->on_pbCopyToCP_clicked(); break;
        case 13: _t->on_pbPasteFromCB_clicked(); break;
        case 14: _t->on_actionAbout_translations_triggered(); break;
        case 15: _t->on_pbTEXChooseStyleIncFile_clicked(); break;
        case 16: _t->on_pbLATEXChooseStyleIncFile_clicked(); break;
        case 17: _t->on_pbHTMLChooseStyleIncFile_clicked(); break;
        case 18: _t->on_pbSVGChooseStyleIncFile_clicked(); break;
        case 19: _t->on_pbCopyAndPaste_clicked(); break;
        case 20: _t->on_pbLSInitialize_clicked(); break;
        case 21: _t->on_leLSExec_textChanged(); break;
        case 22: _t->on_pbSelWorkspace_clicked(); break;
        case 23: _t->on_pbSelExecutable_clicked(); break;
        case 24: _t->on_action_License_triggered(); break;
        case 25: _t->on_action_Changelog_triggered(); break;
        case 26: _t->on_action_Plug_Ins_triggered(); break;
        case 27: _t->on_action_Manual_triggered(); break;
        case 28: _t->on_action_About_Highlight_triggered(); break;
        case 29: _t->on_action_Exit_triggered(); break;
        case 30: _t->plausibility(); break;
        case 31: _t->updatePreview(); break;
        case 32: _t->openFiles(); break;
        case 33: _t->loadLSProfile(); break;
        case 34: _t->on_pbPluginReadFilePath_clicked(); break;
        case 35: _t->on_pbBrowseOutDir_clicked(); break;
        case 36: _t->on_lvUserScripts_itemClicked((*reinterpret_cast<std::add_pointer_t<QListWidgetItem*>>(_a[1]))); break;
        case 37: _t->on_lvPluginScripts_itemClicked((*reinterpret_cast<std::add_pointer_t<QListWidgetItem*>>(_a[1]))); break;
        case 38: _t->on_browserPreview_selectionChanged(); break;
        case 39: _t->on_comboThemeFilter_currentIndexChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        default: ;
        }
    }
}

const QMetaObject *MainWindow::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *MainWindow::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10MainWindowE_t>.strings))
        return static_cast<void*>(this);
    return QMainWindow::qt_metacast(_clname);
}

int MainWindow::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QMainWindow::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 40)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 40;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 40)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 40;
    }
    return _id;
}
QT_WARNING_POP
