/********************************************************************************
** Form generated from reading UI file 'syntax_chooser.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_SYNTAX_CHOOSER_H
#define UI_SYNTAX_CHOOSER_H

#include <QtCore/QVariant>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QVBoxLayout>

QT_BEGIN_NAMESPACE

class Ui_syntax_chooser
{
public:
    QVBoxLayout *verticalLayout_2;
    QGroupBox *groupBox;
    QVBoxLayout *verticalLayout;
    QLabel *lblDescription;
    QHBoxLayout *horizontalLayout;
    QComboBox *cbSyntaxNames;
    QCheckBox *cbRemember;
    QDialogButtonBox *buttonBox;

    void setupUi(QDialog *syntax_chooser)
    {
        if (syntax_chooser->objectName().isEmpty())
            syntax_chooser->setObjectName("syntax_chooser");
        syntax_chooser->resize(392, 197);
        verticalLayout_2 = new QVBoxLayout(syntax_chooser);
        verticalLayout_2->setObjectName("verticalLayout_2");
        groupBox = new QGroupBox(syntax_chooser);
        groupBox->setObjectName("groupBox");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Preferred);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(groupBox->sizePolicy().hasHeightForWidth());
        groupBox->setSizePolicy(sizePolicy);
        groupBox->setMaximumSize(QSize(16777215, 200));
        verticalLayout = new QVBoxLayout(groupBox);
        verticalLayout->setObjectName("verticalLayout");
        lblDescription = new QLabel(groupBox);
        lblDescription->setObjectName("lblDescription");
        lblDescription->setMinimumSize(QSize(0, 40));
        lblDescription->setBaseSize(QSize(0, 40));
        lblDescription->setTextFormat(Qt::PlainText);
        lblDescription->setScaledContents(false);
        lblDescription->setWordWrap(true);

        verticalLayout->addWidget(lblDescription);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        cbSyntaxNames = new QComboBox(groupBox);
        cbSyntaxNames->setObjectName("cbSyntaxNames");
        sizePolicy.setHeightForWidth(cbSyntaxNames->sizePolicy().hasHeightForWidth());
        cbSyntaxNames->setSizePolicy(sizePolicy);

        horizontalLayout->addWidget(cbSyntaxNames);

        cbRemember = new QCheckBox(groupBox);
        cbRemember->setObjectName("cbRemember");

        horizontalLayout->addWidget(cbRemember);


        verticalLayout->addLayout(horizontalLayout);


        verticalLayout_2->addWidget(groupBox);

        buttonBox = new QDialogButtonBox(syntax_chooser);
        buttonBox->setObjectName("buttonBox");
        buttonBox->setAutoFillBackground(false);
        buttonBox->setOrientation(Qt::Horizontal);
        buttonBox->setStandardButtons(QDialogButtonBox::Ok);

        verticalLayout_2->addWidget(buttonBox);


        retranslateUi(syntax_chooser);
        QObject::connect(buttonBox, &QDialogButtonBox::accepted, syntax_chooser, qOverload<>(&QDialog::accept));
        QObject::connect(buttonBox, &QDialogButtonBox::rejected, syntax_chooser, qOverload<>(&QDialog::reject));

        QMetaObject::connectSlotsByName(syntax_chooser);
    } // setupUi

    void retranslateUi(QDialog *syntax_chooser)
    {
        syntax_chooser->setWindowTitle(QCoreApplication::translate("syntax_chooser", "Syntax selection", nullptr));
        groupBox->setTitle(QCoreApplication::translate("syntax_chooser", "Select correct syntax", nullptr));
        lblDescription->setText(QCoreApplication::translate("syntax_chooser", "TextLabel", nullptr));
#if QT_CONFIG(tooltip)
        cbSyntaxNames->setToolTip(QCoreApplication::translate("syntax_chooser", "These entries are configured in filetypes.conf.", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        cbRemember->setToolTip(QCoreApplication::translate("syntax_chooser", "The selection will be remembered for this session.", nullptr));
#endif // QT_CONFIG(tooltip)
        cbRemember->setText(QCoreApplication::translate("syntax_chooser", "Remember selection", nullptr));
    } // retranslateUi

};

namespace Ui {
    class syntax_chooser: public Ui_syntax_chooser {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_SYNTAX_CHOOSER_H
