/********************************************************************************
** Form generated from reading UI file 'mainwindow.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_MAINWINDOW_H
#define UI_MAINWINDOW_H

#include <QtCore/QVariant>
#include <QtGui/QAction>
#include <QtGui/QIcon>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QDockWidget>
#include <QtWidgets/QFontComboBox>
#include <QtWidgets/QFrame>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QListWidget>
#include <QtWidgets/QMainWindow>
#include <QtWidgets/QMenu>
#include <QtWidgets/QMenuBar>
#include <QtWidgets/QProgressBar>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QSpinBox>
#include <QtWidgets/QStackedWidget>
#include <QtWidgets/QStatusBar>
#include <QtWidgets/QTabWidget>
#include <QtWidgets/QTextBrowser>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_MainWindowClass
{
public:
    QAction *action_Open_files;
    QAction *action_Exit;
    QAction *action_Load;
    QAction *action_Save;
    QAction *actionLoad_default_project;
    QAction *action_Manual;
    QAction *action_Tips;
    QAction *action_Changelog;
    QAction *action_License;
    QAction *action_About_Highlight;
    QAction *actionAbout_translations;
    QAction *actionVisit_website;
    QAction *actionDock_floating_panels;
    QAction *action_Plug_Ins;
    QAction *action_File_access_trace_W32;
    QWidget *centralWidget;
    QVBoxLayout *verticalLayout_17;
    QVBoxLayout *verticalLayout_13;
    QLabel *lbPreview;
    QTextBrowser *browserPreview;
    QHBoxLayout *horizontalLayout_13;
    QLabel *label_14;
    QProgressBar *progressBar;
    QMenuBar *menuBar;
    QMenu *menu_File;
    QMenu *menu_Help;
    QStatusBar *statusBar;
    QDockWidget *dockWidget;
    QWidget *dockWidgetContents;
    QVBoxLayout *verticalLayout_16;
    QTabWidget *tabIOSelection;
    QWidget *tab_file_io;
    QVBoxLayout *verticalLayout_3;
    QPushButton *pbOpenFiles;
    QListWidget *lvInputFiles;
    QHBoxLayout *horizontalLayout;
    QPushButton *pbClearSelection;
    QPushButton *pbClearAll;
    QFrame *line_11;
    QVBoxLayout *verticalLayout;
    QLabel *label_11;
    QHBoxLayout *horizontalLayout_22;
    QLineEdit *leOutputDest;
    QPushButton *pbOutputDest;
    QPushButton *pbBrowseOutDir;
    QCheckBox *cbWrite2Src;
    QFrame *line_2;
    QPushButton *pbStartConversion;
    QPushButton *pbCopyFile2CP;
    QWidget *tab_clipboard;
    QVBoxLayout *verticalLayout_15;
    QHBoxLayout *horizontalLayout_23;
    QLabel *label_13;
    QComboBox *comboSelectSyntax;
    QPushButton *pbPasteFromCB;
    QPushButton *pbCopyToCP;
    QSpacerItem *verticalSpacer_7;
    QPushButton *pbCopyAndPaste;
    QSpacerItem *verticalSpacer_8;
    QCheckBox *cbCopyRange;
    QSpacerItem *verticalSpacer_12;
    QWidget *tab_plugins;
    QVBoxLayout *verticalLayout_7;
    QPushButton *pbSelectPlugin;
    QListWidget *lvPluginScripts;
    QHBoxLayout *horizontalLayout_24;
    QPushButton *pbClearSelPlugin;
    QPushButton *pbClearAllPlugins;
    QGroupBox *groupPluginDescription;
    QGridLayout *gridLayout_3;
    QLabel *lblPluginDescription;
    QLabel *label_15;
    QHBoxLayout *horizontalLayout_29;
    QLineEdit *lePluginReadFilePath;
    QPushButton *pbPluginReadFilePath;
    QWidget *tab_scripts;
    QVBoxLayout *verticalLayout_18;
    QPushButton *pbSelectScript;
    QListWidget *lvUserScripts;
    QHBoxLayout *horizontalLayout_25;
    QPushButton *pbClearSelScript;
    QPushButton *pbClearAllScripts;
    QGroupBox *groupScriptDescription;
    QGridLayout *gridLayout_4;
    QLabel *lblScriptDescription;
    QWidget *tab_lsp;
    QGridLayout *gridLayout_5;
    QHBoxLayout *horizontalLayout_32;
    QLabel *label_17;
    QLineEdit *leLSExec;
    QPushButton *pbSelExecutable;
    QHBoxLayout *horizontalLayout_31;
    QLabel *label_10;
    QComboBox *comboLSProfiles;
    QHBoxLayout *horizontalLayout_34;
    QCheckBox *cbLSDebug;
    QPushButton *pbLSInitialize;
    QHBoxLayout *horizontalLayout_33;
    QLabel *label_16;
    QLineEdit *leLSWorkspace;
    QPushButton *pbSelWorkspace;
    QFrame *line_6;
    QCheckBox *cbLSSemantic;
    QCheckBox *cbLSSyntaxErrors;
    QCheckBox *cbLSHover;
    QCheckBox *cbLSLegacy;
    QTabWidget *tabWidgetOptions;
    QWidget *page_general;
    QVBoxLayout *verticalLayout_2;
    QHBoxLayout *horizontalLayout_8;
    QLabel *label_2;
    QComboBox *comboFormat;
    QFrame *line_9;
    QHBoxLayout *horizontalLayout_21;
    QCheckBox *cbIncLineNo;
    QSpacerItem *horizontalSpacer_4;
    QSpinBox *sbLineNoStart;
    QSpinBox *sbLineNoWidth;
    QCheckBox *cbPadZeroes;
    QCheckBox *cbFragment;
    QCheckBox *cbKeepInjections;
    QCheckBox *cbValidateInput;
    QCheckBox *cbOmitVersionInfo;
    QHBoxLayout *horizontalLayout_2;
    QCheckBox *cbEncoding;
    QComboBox *comboEncoding;
    QSpacerItem *verticalSpacer_2;
    QWidget *page_output_spec;
    QGridLayout *gridLayout;
    QStackedWidget *stackedSpecificOptions;
    QWidget *HTMLOptions;
    QVBoxLayout *verticalLayout_5;
    QTabWidget *tabWidget;
    QWidget *tabWidgetPage1;
    QVBoxLayout *verticalLayout_10;
    QHBoxLayout *horizontalLayout_10;
    QCheckBox *cbHTMLEmbedStyle;
    QCheckBox *cbHTMLInlineCSS;
    QHBoxLayout *horizontalLayout_14;
    QLabel *label;
    QLineEdit *leHTMLCssPrefix;
    QHBoxLayout *horizontalLayout_9;
    QLabel *label_19;
    QLineEdit *leHTMLStyleFile;
    QHBoxLayout *horizontalLayout_30;
    QLabel *label_20;
    QSpacerItem *horizontalSpacer_5;
    QHBoxLayout *horizontalLayout_7;
    QLineEdit *leHTMLStyleIncFile;
    QPushButton *pbHTMLChooseStyleIncFile;
    QSpacerItem *verticalSpacer_9;
    QWidget *tabWidgetPage3;
    QVBoxLayout *verticalLayout_4;
    QLabel *label_8;
    QCheckBox *cbHTMLAnchors;
    QCheckBox *cbHTMLFileNameAnchor;
    QCheckBox *cbHTMLOrderedList;
    QFrame *line;
    QCheckBox *cbHTMLIndex;
    QCheckBox *cbHTMLEnclosePreTags;
    QFrame *line_5;
    QCheckBox *cbHTMLPasteMIME;
    QSpacerItem *verticalSpacer;
    QWidget *LATEXOptions;
    QVBoxLayout *verticalLayout_12;
    QCheckBox *cbLATEXEscQuotes;
    QCheckBox *cbLATEXBabel;
    QCheckBox *cbLATEXBeamer;
    QCheckBox *cbLATEXPrettySymbols;
    QFrame *line_4;
    QCheckBox *cbLATEXEmbedStyle;
    QHBoxLayout *horizontalLayout_17;
    QLabel *label_21;
    QLineEdit *leLATEXStyleFile;
    QHBoxLayout *horizontalLayout_15;
    QLabel *label_22;
    QSpacerItem *horizontalSpacer_2;
    QHBoxLayout *horizontalLayout_16;
    QLineEdit *leLATEXStyleIncFile;
    QPushButton *pbLATEXChooseStyleIncFile;
    QSpacerItem *verticalSpacer_4;
    QWidget *TEXOptions;
    QVBoxLayout *verticalLayout_14;
    QCheckBox *cbTEXEmbedStyle;
    QHBoxLayout *horizontalLayout_18;
    QLabel *label_24;
    QLineEdit *leTEXStyleFile;
    QHBoxLayout *horizontalLayout_19;
    QLabel *label_25;
    QSpacerItem *horizontalSpacer_3;
    QHBoxLayout *horizontalLayout_20;
    QLineEdit *leTEXStyleIncFile;
    QPushButton *pbTEXChooseStyleIncFile;
    QSpacerItem *verticalSpacer_5;
    QWidget *RTFOptions;
    QVBoxLayout *verticalLayout_9;
    QCheckBox *cbRTFCharStyles;
    QCheckBox *cbRTFPageColor;
    QHBoxLayout *horizontalLayout_11;
    QLabel *label_9;
    QComboBox *comboRTFPageSize;
    QSpacerItem *verticalSpacer_6;
    QWidget *SVGOptions;
    QVBoxLayout *verticalLayout_8;
    QHBoxLayout *horizontalLayout_12;
    QLabel *label_3;
    QLineEdit *leSVGWidth;
    QSpacerItem *horizontalSpacer;
    QLabel *label_12;
    QLineEdit *leSVGHeight;
    QFrame *line_3;
    QCheckBox *cbSVGEmbedStyle;
    QHBoxLayout *horizontalLayout_26;
    QLabel *label_29;
    QLineEdit *leSVGStyleFile;
    QHBoxLayout *horizontalLayout_27;
    QLabel *label_30;
    QSpacerItem *horizontalSpacer_6;
    QHBoxLayout *horizontalLayout_28;
    QLineEdit *leSVGStyleIncFile;
    QPushButton *pbSVGChooseStyleIncFile;
    QSpacerItem *verticalSpacer_10;
    QWidget *EmptyPage;
    QVBoxLayout *verticalLayout_11;
    QLabel *label_7;
    QSpacerItem *verticalSpacer_3;
    QWidget *page_formatting;
    QVBoxLayout *verticalLayout_6;
    QHBoxLayout *horizontalLayout_3;
    QLabel *label_5;
    QSpacerItem *horizontalSpacer_7;
    QComboBox *comboThemeFilter;
    QComboBox *comboTheme;
    QHBoxLayout *horizontalLayout_4;
    QCheckBox *cbReformat;
    QComboBox *comboReformat;
    QHBoxLayout *horizontalLayout_5;
    QCheckBox *cbKwCase;
    QComboBox *comboKwCase;
    QHBoxLayout *horizontalLayout_6;
    QLabel *label_6;
    QSpinBox *sbTabWidth;
    QGridLayout *gridLayout_2;
    QCheckBox *cbWrapping;
    QSpinBox *sbLineLength;
    QCheckBox *cbAdvWrapping;
    QCheckBox *cbOmitWrappedLineNumbers;
    QGridLayout *gridLayout_9;
    QLabel *label_23;
    QFontComboBox *comboFontName;
    QLabel *label_4;
    QLineEdit *leFontSize;

    void setupUi(QMainWindow *MainWindowClass)
    {
        if (MainWindowClass->objectName().isEmpty())
            MainWindowClass->setObjectName("MainWindowClass");
        MainWindowClass->resize(957, 860);
        MainWindowClass->setAcceptDrops(false);
        QIcon icon;
        icon.addFile(QString::fromUtf8(":/highlight.png"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        MainWindowClass->setWindowIcon(icon);
        action_Open_files = new QAction(MainWindowClass);
        action_Open_files->setObjectName("action_Open_files");
        action_Exit = new QAction(MainWindowClass);
        action_Exit->setObjectName("action_Exit");
        action_Load = new QAction(MainWindowClass);
        action_Load->setObjectName("action_Load");
        action_Save = new QAction(MainWindowClass);
        action_Save->setObjectName("action_Save");
        actionLoad_default_project = new QAction(MainWindowClass);
        actionLoad_default_project->setObjectName("actionLoad_default_project");
        action_Manual = new QAction(MainWindowClass);
        action_Manual->setObjectName("action_Manual");
        action_Tips = new QAction(MainWindowClass);
        action_Tips->setObjectName("action_Tips");
        action_Tips->setVisible(false);
        action_Changelog = new QAction(MainWindowClass);
        action_Changelog->setObjectName("action_Changelog");
        action_License = new QAction(MainWindowClass);
        action_License->setObjectName("action_License");
        action_About_Highlight = new QAction(MainWindowClass);
        action_About_Highlight->setObjectName("action_About_Highlight");
        actionAbout_translations = new QAction(MainWindowClass);
        actionAbout_translations->setObjectName("actionAbout_translations");
        actionVisit_website = new QAction(MainWindowClass);
        actionVisit_website->setObjectName("actionVisit_website");
        actionDock_floating_panels = new QAction(MainWindowClass);
        actionDock_floating_panels->setObjectName("actionDock_floating_panels");
        actionDock_floating_panels->setCheckable(true);
        actionDock_floating_panels->setChecked(false);
        action_Plug_Ins = new QAction(MainWindowClass);
        action_Plug_Ins->setObjectName("action_Plug_Ins");
        action_File_access_trace_W32 = new QAction(MainWindowClass);
        action_File_access_trace_W32->setObjectName("action_File_access_trace_W32");
        centralWidget = new QWidget(MainWindowClass);
        centralWidget->setObjectName("centralWidget");
        verticalLayout_17 = new QVBoxLayout(centralWidget);
        verticalLayout_17->setSpacing(6);
        verticalLayout_17->setContentsMargins(11, 11, 11, 11);
        verticalLayout_17->setObjectName("verticalLayout_17");
        verticalLayout_13 = new QVBoxLayout();
        verticalLayout_13->setSpacing(6);
        verticalLayout_13->setObjectName("verticalLayout_13");
        lbPreview = new QLabel(centralWidget);
        lbPreview->setObjectName("lbPreview");

        verticalLayout_13->addWidget(lbPreview);

        browserPreview = new QTextBrowser(centralWidget);
        browserPreview->setObjectName("browserPreview");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Expanding);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(browserPreview->sizePolicy().hasHeightForWidth());
        browserPreview->setSizePolicy(sizePolicy);
        browserPreview->setOpenExternalLinks(true);

        verticalLayout_13->addWidget(browserPreview);

        horizontalLayout_13 = new QHBoxLayout();
        horizontalLayout_13->setSpacing(6);
        horizontalLayout_13->setObjectName("horizontalLayout_13");
        label_14 = new QLabel(centralWidget);
        label_14->setObjectName("label_14");

        horizontalLayout_13->addWidget(label_14);

        progressBar = new QProgressBar(centralWidget);
        progressBar->setObjectName("progressBar");
        progressBar->setValue(0);

        horizontalLayout_13->addWidget(progressBar);


        verticalLayout_13->addLayout(horizontalLayout_13);


        verticalLayout_17->addLayout(verticalLayout_13);

        MainWindowClass->setCentralWidget(centralWidget);
        menuBar = new QMenuBar(MainWindowClass);
        menuBar->setObjectName("menuBar");
        menuBar->setGeometry(QRect(0, 0, 957, 20));
        menu_File = new QMenu(menuBar);
        menu_File->setObjectName("menu_File");
        menu_Help = new QMenu(menuBar);
        menu_Help->setObjectName("menu_Help");
        MainWindowClass->setMenuBar(menuBar);
        statusBar = new QStatusBar(MainWindowClass);
        statusBar->setObjectName("statusBar");
        MainWindowClass->setStatusBar(statusBar);
        dockWidget = new QDockWidget(MainWindowClass);
        dockWidget->setObjectName("dockWidget");
        dockWidget->setFloating(false);
        dockWidget->setFeatures(QDockWidget::DockWidgetMovable);
        dockWidget->setAllowedAreas(Qt::LeftDockWidgetArea|Qt::RightDockWidgetArea);
        dockWidgetContents = new QWidget();
        dockWidgetContents->setObjectName("dockWidgetContents");
        verticalLayout_16 = new QVBoxLayout(dockWidgetContents);
        verticalLayout_16->setSpacing(6);
        verticalLayout_16->setContentsMargins(11, 11, 11, 11);
        verticalLayout_16->setObjectName("verticalLayout_16");
        tabIOSelection = new QTabWidget(dockWidgetContents);
        tabIOSelection->setObjectName("tabIOSelection");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(tabIOSelection->sizePolicy().hasHeightForWidth());
        tabIOSelection->setSizePolicy(sizePolicy1);
        tabIOSelection->setMinimumSize(QSize(0, 0));
        tabIOSelection->setElideMode(Qt::ElideNone);
        tabIOSelection->setUsesScrollButtons(false);
        tabIOSelection->setDocumentMode(false);
        tabIOSelection->setTabsClosable(false);
        tabIOSelection->setMovable(false);
        tab_file_io = new QWidget();
        tab_file_io->setObjectName("tab_file_io");
        verticalLayout_3 = new QVBoxLayout(tab_file_io);
        verticalLayout_3->setSpacing(6);
        verticalLayout_3->setContentsMargins(11, 11, 11, 11);
        verticalLayout_3->setObjectName("verticalLayout_3");
        pbOpenFiles = new QPushButton(tab_file_io);
        pbOpenFiles->setObjectName("pbOpenFiles");
        QIcon icon1;
        icon1.addFile(QString::fromUtf8(":/file.png"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        pbOpenFiles->setIcon(icon1);

        verticalLayout_3->addWidget(pbOpenFiles);

        lvInputFiles = new QListWidget(tab_file_io);
        lvInputFiles->setObjectName("lvInputFiles");
        QSizePolicy sizePolicy2(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::MinimumExpanding);
        sizePolicy2.setHorizontalStretch(0);
        sizePolicy2.setVerticalStretch(0);
        sizePolicy2.setHeightForWidth(lvInputFiles->sizePolicy().hasHeightForWidth());
        lvInputFiles->setSizePolicy(sizePolicy2);
        lvInputFiles->setMinimumSize(QSize(0, 50));
        lvInputFiles->setAcceptDrops(false);
        lvInputFiles->setDragDropMode(QAbstractItemView::DropOnly);
        lvInputFiles->setAlternatingRowColors(true);
        lvInputFiles->setSelectionMode(QAbstractItemView::ExtendedSelection);
        lvInputFiles->setSelectionBehavior(QAbstractItemView::SelectRows);
        lvInputFiles->setSortingEnabled(true);

        verticalLayout_3->addWidget(lvInputFiles);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setSpacing(6);
        horizontalLayout->setObjectName("horizontalLayout");
        pbClearSelection = new QPushButton(tab_file_io);
        pbClearSelection->setObjectName("pbClearSelection");

        horizontalLayout->addWidget(pbClearSelection);

        pbClearAll = new QPushButton(tab_file_io);
        pbClearAll->setObjectName("pbClearAll");

        horizontalLayout->addWidget(pbClearAll);


        verticalLayout_3->addLayout(horizontalLayout);

        line_11 = new QFrame(tab_file_io);
        line_11->setObjectName("line_11");
        line_11->setFrameShape(QFrame::Shape::HLine);
        line_11->setFrameShadow(QFrame::Shadow::Sunken);

        verticalLayout_3->addWidget(line_11);

        verticalLayout = new QVBoxLayout();
        verticalLayout->setSpacing(6);
        verticalLayout->setObjectName("verticalLayout");
        label_11 = new QLabel(tab_file_io);
        label_11->setObjectName("label_11");
        QFont font;
        font.setBold(true);
        label_11->setFont(font);

        verticalLayout->addWidget(label_11);

        horizontalLayout_22 = new QHBoxLayout();
        horizontalLayout_22->setSpacing(6);
        horizontalLayout_22->setObjectName("horizontalLayout_22");
        leOutputDest = new QLineEdit(tab_file_io);
        leOutputDest->setObjectName("leOutputDest");
        QSizePolicy sizePolicy3(QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Preferred);
        sizePolicy3.setHorizontalStretch(0);
        sizePolicy3.setVerticalStretch(0);
        sizePolicy3.setHeightForWidth(leOutputDest->sizePolicy().hasHeightForWidth());
        leOutputDest->setSizePolicy(sizePolicy3);

        horizontalLayout_22->addWidget(leOutputDest);

        pbOutputDest = new QPushButton(tab_file_io);
        pbOutputDest->setObjectName("pbOutputDest");
        QSizePolicy sizePolicy4(QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Fixed);
        sizePolicy4.setHorizontalStretch(0);
        sizePolicy4.setVerticalStretch(0);
        sizePolicy4.setHeightForWidth(pbOutputDest->sizePolicy().hasHeightForWidth());
        pbOutputDest->setSizePolicy(sizePolicy4);
        pbOutputDest->setMinimumSize(QSize(20, 0));

        horizontalLayout_22->addWidget(pbOutputDest);

        pbBrowseOutDir = new QPushButton(tab_file_io);
        pbBrowseOutDir->setObjectName("pbBrowseOutDir");
        sizePolicy4.setHeightForWidth(pbBrowseOutDir->sizePolicy().hasHeightForWidth());
        pbBrowseOutDir->setSizePolicy(sizePolicy4);
        pbBrowseOutDir->setMinimumSize(QSize(20, 0));
        QIcon icon2;
        icon2.addFile(QString::fromUtf8(":/folder.png"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        pbBrowseOutDir->setIcon(icon2);

        horizontalLayout_22->addWidget(pbBrowseOutDir);


        verticalLayout->addLayout(horizontalLayout_22);

        cbWrite2Src = new QCheckBox(tab_file_io);
        cbWrite2Src->setObjectName("cbWrite2Src");

        verticalLayout->addWidget(cbWrite2Src);


        verticalLayout_3->addLayout(verticalLayout);

        line_2 = new QFrame(tab_file_io);
        line_2->setObjectName("line_2");
        line_2->setFrameShape(QFrame::Shape::HLine);
        line_2->setFrameShadow(QFrame::Shadow::Sunken);

        verticalLayout_3->addWidget(line_2);

        pbStartConversion = new QPushButton(tab_file_io);
        pbStartConversion->setObjectName("pbStartConversion");
        QSizePolicy sizePolicy5(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Fixed);
        sizePolicy5.setHorizontalStretch(0);
        sizePolicy5.setVerticalStretch(0);
        sizePolicy5.setHeightForWidth(pbStartConversion->sizePolicy().hasHeightForWidth());
        pbStartConversion->setSizePolicy(sizePolicy5);
        pbStartConversion->setMinimumSize(QSize(120, 0));
        QFont font1;
        font1.setPointSize(11);
        font1.setBold(true);
        pbStartConversion->setFont(font1);

        verticalLayout_3->addWidget(pbStartConversion);

        pbCopyFile2CP = new QPushButton(tab_file_io);
        pbCopyFile2CP->setObjectName("pbCopyFile2CP");

        verticalLayout_3->addWidget(pbCopyFile2CP);

        tabIOSelection->addTab(tab_file_io, icon1, QString());
        tab_clipboard = new QWidget();
        tab_clipboard->setObjectName("tab_clipboard");
        verticalLayout_15 = new QVBoxLayout(tab_clipboard);
        verticalLayout_15->setSpacing(6);
        verticalLayout_15->setContentsMargins(11, 11, 11, 11);
        verticalLayout_15->setObjectName("verticalLayout_15");
        horizontalLayout_23 = new QHBoxLayout();
        horizontalLayout_23->setSpacing(6);
        horizontalLayout_23->setObjectName("horizontalLayout_23");
        label_13 = new QLabel(tab_clipboard);
        label_13->setObjectName("label_13");

        horizontalLayout_23->addWidget(label_13);

        comboSelectSyntax = new QComboBox(tab_clipboard);
        comboSelectSyntax->setObjectName("comboSelectSyntax");

        horizontalLayout_23->addWidget(comboSelectSyntax);


        verticalLayout_15->addLayout(horizontalLayout_23);

        pbPasteFromCB = new QPushButton(tab_clipboard);
        pbPasteFromCB->setObjectName("pbPasteFromCB");
        QIcon icon3;
        icon3.addFile(QString::fromUtf8(":/clipboard.png"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        pbPasteFromCB->setIcon(icon3);

        verticalLayout_15->addWidget(pbPasteFromCB);

        pbCopyToCP = new QPushButton(tab_clipboard);
        pbCopyToCP->setObjectName("pbCopyToCP");
        pbCopyToCP->setFont(font);

        verticalLayout_15->addWidget(pbCopyToCP);

        verticalSpacer_7 = new QSpacerItem(20, 10, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Minimum);

        verticalLayout_15->addItem(verticalSpacer_7);

        pbCopyAndPaste = new QPushButton(tab_clipboard);
        pbCopyAndPaste->setObjectName("pbCopyAndPaste");

        verticalLayout_15->addWidget(pbCopyAndPaste);

        verticalSpacer_8 = new QSpacerItem(20, 10, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Minimum);

        verticalLayout_15->addItem(verticalSpacer_8);

        cbCopyRange = new QCheckBox(tab_clipboard);
        cbCopyRange->setObjectName("cbCopyRange");
        cbCopyRange->setEnabled(false);

        verticalLayout_15->addWidget(cbCopyRange);

        verticalSpacer_12 = new QSpacerItem(20, 106, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_15->addItem(verticalSpacer_12);

        tabIOSelection->addTab(tab_clipboard, icon3, QString());
        tab_plugins = new QWidget();
        tab_plugins->setObjectName("tab_plugins");
        verticalLayout_7 = new QVBoxLayout(tab_plugins);
        verticalLayout_7->setSpacing(6);
        verticalLayout_7->setContentsMargins(11, 11, 11, 11);
        verticalLayout_7->setObjectName("verticalLayout_7");
        pbSelectPlugin = new QPushButton(tab_plugins);
        pbSelectPlugin->setObjectName("pbSelectPlugin");
        QIcon icon4;
        icon4.addFile(QString::fromUtf8(":/plugin.png"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        pbSelectPlugin->setIcon(icon4);

        verticalLayout_7->addWidget(pbSelectPlugin);

        lvPluginScripts = new QListWidget(tab_plugins);
        lvPluginScripts->setObjectName("lvPluginScripts");
        lvPluginScripts->setAlternatingRowColors(true);
        lvPluginScripts->setSelectionMode(QAbstractItemView::SingleSelection);
        lvPluginScripts->setSortingEnabled(true);

        verticalLayout_7->addWidget(lvPluginScripts);

        horizontalLayout_24 = new QHBoxLayout();
        horizontalLayout_24->setSpacing(6);
        horizontalLayout_24->setObjectName("horizontalLayout_24");
        pbClearSelPlugin = new QPushButton(tab_plugins);
        pbClearSelPlugin->setObjectName("pbClearSelPlugin");

        horizontalLayout_24->addWidget(pbClearSelPlugin);

        pbClearAllPlugins = new QPushButton(tab_plugins);
        pbClearAllPlugins->setObjectName("pbClearAllPlugins");

        horizontalLayout_24->addWidget(pbClearAllPlugins);


        verticalLayout_7->addLayout(horizontalLayout_24);

        groupPluginDescription = new QGroupBox(tab_plugins);
        groupPluginDescription->setObjectName("groupPluginDescription");
        gridLayout_3 = new QGridLayout(groupPluginDescription);
        gridLayout_3->setSpacing(6);
        gridLayout_3->setContentsMargins(11, 11, 11, 11);
        gridLayout_3->setObjectName("gridLayout_3");
        lblPluginDescription = new QLabel(groupPluginDescription);
        lblPluginDescription->setObjectName("lblPluginDescription");
        lblPluginDescription->setMaximumSize(QSize(16777215, 16777215));
        lblPluginDescription->setTextFormat(Qt::PlainText);
        lblPluginDescription->setWordWrap(true);
        lblPluginDescription->setMargin(2);
        lblPluginDescription->setTextInteractionFlags(Qt::NoTextInteraction);

        gridLayout_3->addWidget(lblPluginDescription, 0, 0, 1, 1);


        verticalLayout_7->addWidget(groupPluginDescription);

        label_15 = new QLabel(tab_plugins);
        label_15->setObjectName("label_15");
        label_15->setFont(font);

        verticalLayout_7->addWidget(label_15);

        horizontalLayout_29 = new QHBoxLayout();
        horizontalLayout_29->setSpacing(6);
        horizontalLayout_29->setObjectName("horizontalLayout_29");
        lePluginReadFilePath = new QLineEdit(tab_plugins);
        lePluginReadFilePath->setObjectName("lePluginReadFilePath");

        horizontalLayout_29->addWidget(lePluginReadFilePath);

        pbPluginReadFilePath = new QPushButton(tab_plugins);
        pbPluginReadFilePath->setObjectName("pbPluginReadFilePath");
        pbPluginReadFilePath->setMinimumSize(QSize(20, 0));

        horizontalLayout_29->addWidget(pbPluginReadFilePath);


        verticalLayout_7->addLayout(horizontalLayout_29);

        tabIOSelection->addTab(tab_plugins, icon4, QString());
        tab_scripts = new QWidget();
        tab_scripts->setObjectName("tab_scripts");
        verticalLayout_18 = new QVBoxLayout(tab_scripts);
        verticalLayout_18->setSpacing(6);
        verticalLayout_18->setContentsMargins(11, 11, 11, 11);
        verticalLayout_18->setObjectName("verticalLayout_18");
        pbSelectScript = new QPushButton(tab_scripts);
        pbSelectScript->setObjectName("pbSelectScript");
        QIcon icon5;
        icon5.addFile(QString::fromUtf8(":/script.png"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        pbSelectScript->setIcon(icon5);

        verticalLayout_18->addWidget(pbSelectScript);

        lvUserScripts = new QListWidget(tab_scripts);
        lvUserScripts->setObjectName("lvUserScripts");
        lvUserScripts->setEnabled(true);

        verticalLayout_18->addWidget(lvUserScripts);

        horizontalLayout_25 = new QHBoxLayout();
        horizontalLayout_25->setSpacing(6);
        horizontalLayout_25->setObjectName("horizontalLayout_25");
        pbClearSelScript = new QPushButton(tab_scripts);
        pbClearSelScript->setObjectName("pbClearSelScript");

        horizontalLayout_25->addWidget(pbClearSelScript);

        pbClearAllScripts = new QPushButton(tab_scripts);
        pbClearAllScripts->setObjectName("pbClearAllScripts");

        horizontalLayout_25->addWidget(pbClearAllScripts);


        verticalLayout_18->addLayout(horizontalLayout_25);

        groupScriptDescription = new QGroupBox(tab_scripts);
        groupScriptDescription->setObjectName("groupScriptDescription");
        gridLayout_4 = new QGridLayout(groupScriptDescription);
        gridLayout_4->setSpacing(6);
        gridLayout_4->setContentsMargins(11, 11, 11, 11);
        gridLayout_4->setObjectName("gridLayout_4");
        lblScriptDescription = new QLabel(groupScriptDescription);
        lblScriptDescription->setObjectName("lblScriptDescription");

        gridLayout_4->addWidget(lblScriptDescription, 0, 0, 1, 1);


        verticalLayout_18->addWidget(groupScriptDescription);

        tabIOSelection->addTab(tab_scripts, icon5, QString());
        tab_lsp = new QWidget();
        tab_lsp->setObjectName("tab_lsp");
        gridLayout_5 = new QGridLayout(tab_lsp);
        gridLayout_5->setSpacing(6);
        gridLayout_5->setContentsMargins(11, 11, 11, 11);
        gridLayout_5->setObjectName("gridLayout_5");
        horizontalLayout_32 = new QHBoxLayout();
        horizontalLayout_32->setSpacing(6);
        horizontalLayout_32->setObjectName("horizontalLayout_32");
        label_17 = new QLabel(tab_lsp);
        label_17->setObjectName("label_17");

        horizontalLayout_32->addWidget(label_17);

        leLSExec = new QLineEdit(tab_lsp);
        leLSExec->setObjectName("leLSExec");

        horizontalLayout_32->addWidget(leLSExec);

        pbSelExecutable = new QPushButton(tab_lsp);
        pbSelExecutable->setObjectName("pbSelExecutable");
        pbSelExecutable->setMinimumSize(QSize(20, 0));

        horizontalLayout_32->addWidget(pbSelExecutable);


        gridLayout_5->addLayout(horizontalLayout_32, 1, 0, 1, 1);

        horizontalLayout_31 = new QHBoxLayout();
        horizontalLayout_31->setSpacing(6);
        horizontalLayout_31->setObjectName("horizontalLayout_31");
        label_10 = new QLabel(tab_lsp);
        label_10->setObjectName("label_10");

        horizontalLayout_31->addWidget(label_10);

        comboLSProfiles = new QComboBox(tab_lsp);
        comboLSProfiles->setObjectName("comboLSProfiles");
        comboLSProfiles->setEditable(false);
        comboLSProfiles->setCurrentText(QString::fromUtf8(""));

        horizontalLayout_31->addWidget(comboLSProfiles);


        gridLayout_5->addLayout(horizontalLayout_31, 0, 0, 1, 1);

        horizontalLayout_34 = new QHBoxLayout();
        horizontalLayout_34->setSpacing(6);
        horizontalLayout_34->setObjectName("horizontalLayout_34");
        cbLSDebug = new QCheckBox(tab_lsp);
        cbLSDebug->setObjectName("cbLSDebug");

        horizontalLayout_34->addWidget(cbLSDebug);

        pbLSInitialize = new QPushButton(tab_lsp);
        pbLSInitialize->setObjectName("pbLSInitialize");

        horizontalLayout_34->addWidget(pbLSInitialize);


        gridLayout_5->addLayout(horizontalLayout_34, 2, 0, 1, 1);

        horizontalLayout_33 = new QHBoxLayout();
        horizontalLayout_33->setSpacing(6);
        horizontalLayout_33->setObjectName("horizontalLayout_33");
        label_16 = new QLabel(tab_lsp);
        label_16->setObjectName("label_16");

        horizontalLayout_33->addWidget(label_16);

        leLSWorkspace = new QLineEdit(tab_lsp);
        leLSWorkspace->setObjectName("leLSWorkspace");

        horizontalLayout_33->addWidget(leLSWorkspace);

        pbSelWorkspace = new QPushButton(tab_lsp);
        pbSelWorkspace->setObjectName("pbSelWorkspace");
        pbSelWorkspace->setMinimumSize(QSize(20, 0));

        horizontalLayout_33->addWidget(pbSelWorkspace);


        gridLayout_5->addLayout(horizontalLayout_33, 4, 0, 1, 1);

        line_6 = new QFrame(tab_lsp);
        line_6->setObjectName("line_6");
        line_6->setFrameShape(QFrame::Shape::HLine);
        line_6->setFrameShadow(QFrame::Shadow::Sunken);

        gridLayout_5->addWidget(line_6, 3, 0, 1, 1);

        cbLSSemantic = new QCheckBox(tab_lsp);
        cbLSSemantic->setObjectName("cbLSSemantic");
        cbLSSemantic->setCheckable(true);

        gridLayout_5->addWidget(cbLSSemantic, 6, 0, 1, 1);

        cbLSSyntaxErrors = new QCheckBox(tab_lsp);
        cbLSSyntaxErrors->setObjectName("cbLSSyntaxErrors");
        cbLSSyntaxErrors->setCheckable(true);

        gridLayout_5->addWidget(cbLSSyntaxErrors, 7, 0, 1, 1);

        cbLSHover = new QCheckBox(tab_lsp);
        cbLSHover->setObjectName("cbLSHover");
        cbLSHover->setCheckable(true);

        gridLayout_5->addWidget(cbLSHover, 5, 0, 1, 1);

        cbLSLegacy = new QCheckBox(tab_lsp);
        cbLSLegacy->setObjectName("cbLSLegacy");

        gridLayout_5->addWidget(cbLSLegacy, 8, 0, 1, 1);

        QIcon icon6;
        icon6.addFile(QString::fromUtf8(":/script_blue.png"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        tabIOSelection->addTab(tab_lsp, icon6, QString());

        verticalLayout_16->addWidget(tabIOSelection);

        tabWidgetOptions = new QTabWidget(dockWidgetContents);
        tabWidgetOptions->setObjectName("tabWidgetOptions");
        QSizePolicy sizePolicy6(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Minimum);
        sizePolicy6.setHorizontalStretch(0);
        sizePolicy6.setVerticalStretch(0);
        sizePolicy6.setHeightForWidth(tabWidgetOptions->sizePolicy().hasHeightForWidth());
        tabWidgetOptions->setSizePolicy(sizePolicy6);
        tabWidgetOptions->setMinimumSize(QSize(0, 0));
        tabWidgetOptions->setLayoutDirection(Qt::LeftToRight);
        tabWidgetOptions->setTabPosition(QTabWidget::North);
        tabWidgetOptions->setTabShape(QTabWidget::Rounded);
        tabWidgetOptions->setMovable(false);
        page_general = new QWidget();
        page_general->setObjectName("page_general");
        verticalLayout_2 = new QVBoxLayout(page_general);
        verticalLayout_2->setSpacing(6);
        verticalLayout_2->setContentsMargins(11, 11, 11, 11);
        verticalLayout_2->setObjectName("verticalLayout_2");
        horizontalLayout_8 = new QHBoxLayout();
        horizontalLayout_8->setSpacing(6);
        horizontalLayout_8->setObjectName("horizontalLayout_8");
        label_2 = new QLabel(page_general);
        label_2->setObjectName("label_2");
        QSizePolicy sizePolicy7(QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Minimum);
        sizePolicy7.setHorizontalStretch(0);
        sizePolicy7.setVerticalStretch(0);
        sizePolicy7.setHeightForWidth(label_2->sizePolicy().hasHeightForWidth());
        label_2->setSizePolicy(sizePolicy7);
        label_2->setFont(font);

        horizontalLayout_8->addWidget(label_2);

        comboFormat = new QComboBox(page_general);
        comboFormat->addItem(QString());
        comboFormat->addItem(QString());
        comboFormat->addItem(QString());
        comboFormat->addItem(QString());
        comboFormat->addItem(QString());
        comboFormat->addItem(QString());
        comboFormat->addItem(QString());
        comboFormat->addItem(QString());
        comboFormat->addItem(QString());
        comboFormat->addItem(QString());
        comboFormat->addItem(QString());
        comboFormat->setObjectName("comboFormat");
        sizePolicy5.setHeightForWidth(comboFormat->sizePolicy().hasHeightForWidth());
        comboFormat->setSizePolicy(sizePolicy5);
        comboFormat->setFont(font);

        horizontalLayout_8->addWidget(comboFormat);


        verticalLayout_2->addLayout(horizontalLayout_8);

        line_9 = new QFrame(page_general);
        line_9->setObjectName("line_9");
        line_9->setFrameShape(QFrame::Shape::HLine);
        line_9->setFrameShadow(QFrame::Shadow::Sunken);

        verticalLayout_2->addWidget(line_9);

        horizontalLayout_21 = new QHBoxLayout();
        horizontalLayout_21->setSpacing(6);
        horizontalLayout_21->setObjectName("horizontalLayout_21");
        cbIncLineNo = new QCheckBox(page_general);
        cbIncLineNo->setObjectName("cbIncLineNo");

        horizontalLayout_21->addWidget(cbIncLineNo);

        horizontalSpacer_4 = new QSpacerItem(10, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_21->addItem(horizontalSpacer_4);

        sbLineNoStart = new QSpinBox(page_general);
        sbLineNoStart->setObjectName("sbLineNoStart");
        sbLineNoStart->setButtonSymbols(QAbstractSpinBox::UpDownArrows);
        sbLineNoStart->setAccelerated(true);
        sbLineNoStart->setMaximum(500);
        sbLineNoStart->setValue(1);

        horizontalLayout_21->addWidget(sbLineNoStart);

        sbLineNoWidth = new QSpinBox(page_general);
        sbLineNoWidth->setObjectName("sbLineNoWidth");
        sbLineNoWidth->setMinimum(1);
        sbLineNoWidth->setMaximum(6);
        sbLineNoWidth->setValue(2);

        horizontalLayout_21->addWidget(sbLineNoWidth);


        verticalLayout_2->addLayout(horizontalLayout_21);

        cbPadZeroes = new QCheckBox(page_general);
        cbPadZeroes->setObjectName("cbPadZeroes");
        cbPadZeroes->setEnabled(false);

        verticalLayout_2->addWidget(cbPadZeroes);

        cbFragment = new QCheckBox(page_general);
        cbFragment->setObjectName("cbFragment");
        QSizePolicy sizePolicy8(QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Fixed);
        sizePolicy8.setHorizontalStretch(0);
        sizePolicy8.setVerticalStretch(0);
        sizePolicy8.setHeightForWidth(cbFragment->sizePolicy().hasHeightForWidth());
        cbFragment->setSizePolicy(sizePolicy8);

        verticalLayout_2->addWidget(cbFragment);

        cbKeepInjections = new QCheckBox(page_general);
        cbKeepInjections->setObjectName("cbKeepInjections");

        verticalLayout_2->addWidget(cbKeepInjections);

        cbValidateInput = new QCheckBox(page_general);
        cbValidateInput->setObjectName("cbValidateInput");

        verticalLayout_2->addWidget(cbValidateInput);

        cbOmitVersionInfo = new QCheckBox(page_general);
        cbOmitVersionInfo->setObjectName("cbOmitVersionInfo");

        verticalLayout_2->addWidget(cbOmitVersionInfo);

        horizontalLayout_2 = new QHBoxLayout();
        horizontalLayout_2->setSpacing(6);
        horizontalLayout_2->setObjectName("horizontalLayout_2");
        cbEncoding = new QCheckBox(page_general);
        cbEncoding->setObjectName("cbEncoding");

        horizontalLayout_2->addWidget(cbEncoding);

        comboEncoding = new QComboBox(page_general);
        comboEncoding->addItem(QString());
        comboEncoding->addItem(QString());
        comboEncoding->addItem(QString());
        comboEncoding->addItem(QString());
        comboEncoding->addItem(QString());
        comboEncoding->addItem(QString());
        comboEncoding->addItem(QString());
        comboEncoding->addItem(QString());
        comboEncoding->addItem(QString());
        comboEncoding->addItem(QString());
        comboEncoding->addItem(QString());
        comboEncoding->addItem(QString());
        comboEncoding->addItem(QString());
        comboEncoding->addItem(QString());
        comboEncoding->addItem(QString());
        comboEncoding->addItem(QString());
        comboEncoding->setObjectName("comboEncoding");
        comboEncoding->setEditable(true);
        comboEncoding->setInsertPolicy(QComboBox::InsertAtBottom);

        horizontalLayout_2->addWidget(comboEncoding);


        verticalLayout_2->addLayout(horizontalLayout_2);

        verticalSpacer_2 = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_2->addItem(verticalSpacer_2);

        tabWidgetOptions->addTab(page_general, QString());
        page_output_spec = new QWidget();
        page_output_spec->setObjectName("page_output_spec");
        gridLayout = new QGridLayout(page_output_spec);
        gridLayout->setSpacing(6);
        gridLayout->setContentsMargins(11, 11, 11, 11);
        gridLayout->setObjectName("gridLayout");
        stackedSpecificOptions = new QStackedWidget(page_output_spec);
        stackedSpecificOptions->setObjectName("stackedSpecificOptions");
        QSizePolicy sizePolicy9(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Ignored);
        sizePolicy9.setHorizontalStretch(0);
        sizePolicy9.setVerticalStretch(0);
        sizePolicy9.setHeightForWidth(stackedSpecificOptions->sizePolicy().hasHeightForWidth());
        stackedSpecificOptions->setSizePolicy(sizePolicy9);
        stackedSpecificOptions->setMinimumSize(QSize(0, 0));
        HTMLOptions = new QWidget();
        HTMLOptions->setObjectName("HTMLOptions");
        verticalLayout_5 = new QVBoxLayout(HTMLOptions);
        verticalLayout_5->setSpacing(6);
        verticalLayout_5->setContentsMargins(11, 11, 11, 11);
        verticalLayout_5->setObjectName("verticalLayout_5");
        tabWidget = new QTabWidget(HTMLOptions);
        tabWidget->setObjectName("tabWidget");
        sizePolicy7.setHeightForWidth(tabWidget->sizePolicy().hasHeightForWidth());
        tabWidget->setSizePolicy(sizePolicy7);
        tabWidget->setMinimumSize(QSize(250, 0));
        tabWidgetPage1 = new QWidget();
        tabWidgetPage1->setObjectName("tabWidgetPage1");
        verticalLayout_10 = new QVBoxLayout(tabWidgetPage1);
        verticalLayout_10->setSpacing(6);
        verticalLayout_10->setContentsMargins(11, 11, 11, 11);
        verticalLayout_10->setObjectName("verticalLayout_10");
        horizontalLayout_10 = new QHBoxLayout();
        horizontalLayout_10->setSpacing(6);
        horizontalLayout_10->setObjectName("horizontalLayout_10");
        cbHTMLEmbedStyle = new QCheckBox(tabWidgetPage1);
        cbHTMLEmbedStyle->setObjectName("cbHTMLEmbedStyle");
        cbHTMLEmbedStyle->setChecked(false);

        horizontalLayout_10->addWidget(cbHTMLEmbedStyle);

        cbHTMLInlineCSS = new QCheckBox(tabWidgetPage1);
        cbHTMLInlineCSS->setObjectName("cbHTMLInlineCSS");
        cbHTMLInlineCSS->setEnabled(false);
        cbHTMLInlineCSS->setCheckable(true);

        horizontalLayout_10->addWidget(cbHTMLInlineCSS);


        verticalLayout_10->addLayout(horizontalLayout_10);

        horizontalLayout_14 = new QHBoxLayout();
        horizontalLayout_14->setSpacing(6);
        horizontalLayout_14->setObjectName("horizontalLayout_14");
        label = new QLabel(tabWidgetPage1);
        label->setObjectName("label");

        horizontalLayout_14->addWidget(label);

        leHTMLCssPrefix = new QLineEdit(tabWidgetPage1);
        leHTMLCssPrefix->setObjectName("leHTMLCssPrefix");
        leHTMLCssPrefix->setMaximumSize(QSize(100, 16777215));

        horizontalLayout_14->addWidget(leHTMLCssPrefix);


        verticalLayout_10->addLayout(horizontalLayout_14);

        horizontalLayout_9 = new QHBoxLayout();
        horizontalLayout_9->setSpacing(6);
        horizontalLayout_9->setObjectName("horizontalLayout_9");
        label_19 = new QLabel(tabWidgetPage1);
        label_19->setObjectName("label_19");

        horizontalLayout_9->addWidget(label_19);

        leHTMLStyleFile = new QLineEdit(tabWidgetPage1);
        leHTMLStyleFile->setObjectName("leHTMLStyleFile");
        sizePolicy5.setHeightForWidth(leHTMLStyleFile->sizePolicy().hasHeightForWidth());
        leHTMLStyleFile->setSizePolicy(sizePolicy5);
        leHTMLStyleFile->setMaximumSize(QSize(100, 16777215));

        horizontalLayout_9->addWidget(leHTMLStyleFile);


        verticalLayout_10->addLayout(horizontalLayout_9);

        horizontalLayout_30 = new QHBoxLayout();
        horizontalLayout_30->setSpacing(6);
        horizontalLayout_30->setObjectName("horizontalLayout_30");
        label_20 = new QLabel(tabWidgetPage1);
        label_20->setObjectName("label_20");

        horizontalLayout_30->addWidget(label_20);

        horizontalSpacer_5 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_30->addItem(horizontalSpacer_5);


        verticalLayout_10->addLayout(horizontalLayout_30);

        horizontalLayout_7 = new QHBoxLayout();
        horizontalLayout_7->setSpacing(6);
        horizontalLayout_7->setObjectName("horizontalLayout_7");
        leHTMLStyleIncFile = new QLineEdit(tabWidgetPage1);
        leHTMLStyleIncFile->setObjectName("leHTMLStyleIncFile");
        leHTMLStyleIncFile->setMaximumSize(QSize(16777215, 16777215));

        horizontalLayout_7->addWidget(leHTMLStyleIncFile);

        pbHTMLChooseStyleIncFile = new QPushButton(tabWidgetPage1);
        pbHTMLChooseStyleIncFile->setObjectName("pbHTMLChooseStyleIncFile");
        pbHTMLChooseStyleIncFile->setMinimumSize(QSize(20, 0));

        horizontalLayout_7->addWidget(pbHTMLChooseStyleIncFile);


        verticalLayout_10->addLayout(horizontalLayout_7);

        verticalSpacer_9 = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::MinimumExpanding);

        verticalLayout_10->addItem(verticalSpacer_9);

        tabWidget->addTab(tabWidgetPage1, QString());
        tabWidgetPage3 = new QWidget();
        tabWidgetPage3->setObjectName("tabWidgetPage3");
        verticalLayout_4 = new QVBoxLayout(tabWidgetPage3);
        verticalLayout_4->setSpacing(6);
        verticalLayout_4->setContentsMargins(11, 11, 11, 11);
        verticalLayout_4->setObjectName("verticalLayout_4");
        label_8 = new QLabel(tabWidgetPage3);
        label_8->setObjectName("label_8");
        QFont font2;
        font2.setBold(false);
        label_8->setFont(font2);

        verticalLayout_4->addWidget(label_8);

        cbHTMLAnchors = new QCheckBox(tabWidgetPage3);
        cbHTMLAnchors->setObjectName("cbHTMLAnchors");

        verticalLayout_4->addWidget(cbHTMLAnchors);

        cbHTMLFileNameAnchor = new QCheckBox(tabWidgetPage3);
        cbHTMLFileNameAnchor->setObjectName("cbHTMLFileNameAnchor");

        verticalLayout_4->addWidget(cbHTMLFileNameAnchor);

        cbHTMLOrderedList = new QCheckBox(tabWidgetPage3);
        cbHTMLOrderedList->setObjectName("cbHTMLOrderedList");

        verticalLayout_4->addWidget(cbHTMLOrderedList);

        line = new QFrame(tabWidgetPage3);
        line->setObjectName("line");
        line->setFrameShape(QFrame::Shape::HLine);
        line->setFrameShadow(QFrame::Shadow::Sunken);

        verticalLayout_4->addWidget(line);

        cbHTMLIndex = new QCheckBox(tabWidgetPage3);
        cbHTMLIndex->setObjectName("cbHTMLIndex");

        verticalLayout_4->addWidget(cbHTMLIndex);

        cbHTMLEnclosePreTags = new QCheckBox(tabWidgetPage3);
        cbHTMLEnclosePreTags->setObjectName("cbHTMLEnclosePreTags");

        verticalLayout_4->addWidget(cbHTMLEnclosePreTags);

        line_5 = new QFrame(tabWidgetPage3);
        line_5->setObjectName("line_5");
        line_5->setFrameShape(QFrame::Shape::HLine);
        line_5->setFrameShadow(QFrame::Shadow::Sunken);

        verticalLayout_4->addWidget(line_5);

        cbHTMLPasteMIME = new QCheckBox(tabWidgetPage3);
        cbHTMLPasteMIME->setObjectName("cbHTMLPasteMIME");

        verticalLayout_4->addWidget(cbHTMLPasteMIME);

        verticalSpacer = new QSpacerItem(20, 24, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_4->addItem(verticalSpacer);

        tabWidget->addTab(tabWidgetPage3, QString());

        verticalLayout_5->addWidget(tabWidget);

        stackedSpecificOptions->addWidget(HTMLOptions);
        LATEXOptions = new QWidget();
        LATEXOptions->setObjectName("LATEXOptions");
        verticalLayout_12 = new QVBoxLayout(LATEXOptions);
        verticalLayout_12->setSpacing(6);
        verticalLayout_12->setContentsMargins(11, 11, 11, 11);
        verticalLayout_12->setObjectName("verticalLayout_12");
        cbLATEXEscQuotes = new QCheckBox(LATEXOptions);
        cbLATEXEscQuotes->setObjectName("cbLATEXEscQuotes");

        verticalLayout_12->addWidget(cbLATEXEscQuotes);

        cbLATEXBabel = new QCheckBox(LATEXOptions);
        cbLATEXBabel->setObjectName("cbLATEXBabel");

        verticalLayout_12->addWidget(cbLATEXBabel);

        cbLATEXBeamer = new QCheckBox(LATEXOptions);
        cbLATEXBeamer->setObjectName("cbLATEXBeamer");

        verticalLayout_12->addWidget(cbLATEXBeamer);

        cbLATEXPrettySymbols = new QCheckBox(LATEXOptions);
        cbLATEXPrettySymbols->setObjectName("cbLATEXPrettySymbols");

        verticalLayout_12->addWidget(cbLATEXPrettySymbols);

        line_4 = new QFrame(LATEXOptions);
        line_4->setObjectName("line_4");
        line_4->setFrameShape(QFrame::Shape::HLine);
        line_4->setFrameShadow(QFrame::Shadow::Sunken);

        verticalLayout_12->addWidget(line_4);

        cbLATEXEmbedStyle = new QCheckBox(LATEXOptions);
        cbLATEXEmbedStyle->setObjectName("cbLATEXEmbedStyle");

        verticalLayout_12->addWidget(cbLATEXEmbedStyle);

        horizontalLayout_17 = new QHBoxLayout();
        horizontalLayout_17->setSpacing(6);
        horizontalLayout_17->setObjectName("horizontalLayout_17");
        label_21 = new QLabel(LATEXOptions);
        label_21->setObjectName("label_21");

        horizontalLayout_17->addWidget(label_21);

        leLATEXStyleFile = new QLineEdit(LATEXOptions);
        leLATEXStyleFile->setObjectName("leLATEXStyleFile");
        leLATEXStyleFile->setMaximumSize(QSize(100, 16777215));

        horizontalLayout_17->addWidget(leLATEXStyleFile);


        verticalLayout_12->addLayout(horizontalLayout_17);

        horizontalLayout_15 = new QHBoxLayout();
        horizontalLayout_15->setSpacing(6);
        horizontalLayout_15->setObjectName("horizontalLayout_15");
        label_22 = new QLabel(LATEXOptions);
        label_22->setObjectName("label_22");

        horizontalLayout_15->addWidget(label_22);

        horizontalSpacer_2 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_15->addItem(horizontalSpacer_2);


        verticalLayout_12->addLayout(horizontalLayout_15);

        horizontalLayout_16 = new QHBoxLayout();
        horizontalLayout_16->setSpacing(6);
        horizontalLayout_16->setObjectName("horizontalLayout_16");
        leLATEXStyleIncFile = new QLineEdit(LATEXOptions);
        leLATEXStyleIncFile->setObjectName("leLATEXStyleIncFile");

        horizontalLayout_16->addWidget(leLATEXStyleIncFile);

        pbLATEXChooseStyleIncFile = new QPushButton(LATEXOptions);
        pbLATEXChooseStyleIncFile->setObjectName("pbLATEXChooseStyleIncFile");
        pbLATEXChooseStyleIncFile->setMinimumSize(QSize(20, 0));

        horizontalLayout_16->addWidget(pbLATEXChooseStyleIncFile);


        verticalLayout_12->addLayout(horizontalLayout_16);

        verticalSpacer_4 = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_12->addItem(verticalSpacer_4);

        stackedSpecificOptions->addWidget(LATEXOptions);
        TEXOptions = new QWidget();
        TEXOptions->setObjectName("TEXOptions");
        verticalLayout_14 = new QVBoxLayout(TEXOptions);
        verticalLayout_14->setSpacing(6);
        verticalLayout_14->setContentsMargins(11, 11, 11, 11);
        verticalLayout_14->setObjectName("verticalLayout_14");
        cbTEXEmbedStyle = new QCheckBox(TEXOptions);
        cbTEXEmbedStyle->setObjectName("cbTEXEmbedStyle");

        verticalLayout_14->addWidget(cbTEXEmbedStyle);

        horizontalLayout_18 = new QHBoxLayout();
        horizontalLayout_18->setSpacing(6);
        horizontalLayout_18->setObjectName("horizontalLayout_18");
        label_24 = new QLabel(TEXOptions);
        label_24->setObjectName("label_24");

        horizontalLayout_18->addWidget(label_24);

        leTEXStyleFile = new QLineEdit(TEXOptions);
        leTEXStyleFile->setObjectName("leTEXStyleFile");
        leTEXStyleFile->setMaximumSize(QSize(100, 16777215));

        horizontalLayout_18->addWidget(leTEXStyleFile);


        verticalLayout_14->addLayout(horizontalLayout_18);

        horizontalLayout_19 = new QHBoxLayout();
        horizontalLayout_19->setSpacing(6);
        horizontalLayout_19->setObjectName("horizontalLayout_19");
        label_25 = new QLabel(TEXOptions);
        label_25->setObjectName("label_25");

        horizontalLayout_19->addWidget(label_25);

        horizontalSpacer_3 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_19->addItem(horizontalSpacer_3);


        verticalLayout_14->addLayout(horizontalLayout_19);

        horizontalLayout_20 = new QHBoxLayout();
        horizontalLayout_20->setSpacing(6);
        horizontalLayout_20->setObjectName("horizontalLayout_20");
        leTEXStyleIncFile = new QLineEdit(TEXOptions);
        leTEXStyleIncFile->setObjectName("leTEXStyleIncFile");

        horizontalLayout_20->addWidget(leTEXStyleIncFile);

        pbTEXChooseStyleIncFile = new QPushButton(TEXOptions);
        pbTEXChooseStyleIncFile->setObjectName("pbTEXChooseStyleIncFile");
        pbTEXChooseStyleIncFile->setMinimumSize(QSize(20, 0));

        horizontalLayout_20->addWidget(pbTEXChooseStyleIncFile);


        verticalLayout_14->addLayout(horizontalLayout_20);

        verticalSpacer_5 = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_14->addItem(verticalSpacer_5);

        stackedSpecificOptions->addWidget(TEXOptions);
        RTFOptions = new QWidget();
        RTFOptions->setObjectName("RTFOptions");
        verticalLayout_9 = new QVBoxLayout(RTFOptions);
        verticalLayout_9->setSpacing(6);
        verticalLayout_9->setContentsMargins(11, 11, 11, 11);
        verticalLayout_9->setObjectName("verticalLayout_9");
        cbRTFCharStyles = new QCheckBox(RTFOptions);
        cbRTFCharStyles->setObjectName("cbRTFCharStyles");

        verticalLayout_9->addWidget(cbRTFCharStyles);

        cbRTFPageColor = new QCheckBox(RTFOptions);
        cbRTFPageColor->setObjectName("cbRTFPageColor");

        verticalLayout_9->addWidget(cbRTFPageColor);

        horizontalLayout_11 = new QHBoxLayout();
        horizontalLayout_11->setSpacing(6);
        horizontalLayout_11->setObjectName("horizontalLayout_11");
        label_9 = new QLabel(RTFOptions);
        label_9->setObjectName("label_9");

        horizontalLayout_11->addWidget(label_9);

        comboRTFPageSize = new QComboBox(RTFOptions);
        comboRTFPageSize->addItem(QString());
        comboRTFPageSize->addItem(QString());
        comboRTFPageSize->addItem(QString());
        comboRTFPageSize->addItem(QString());
        comboRTFPageSize->addItem(QString());
        comboRTFPageSize->addItem(QString());
        comboRTFPageSize->addItem(QString());
        comboRTFPageSize->addItem(QString());
        comboRTFPageSize->setObjectName("comboRTFPageSize");
        comboRTFPageSize->setMaximumSize(QSize(80, 16777215));

        horizontalLayout_11->addWidget(comboRTFPageSize);


        verticalLayout_9->addLayout(horizontalLayout_11);

        verticalSpacer_6 = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_9->addItem(verticalSpacer_6);

        stackedSpecificOptions->addWidget(RTFOptions);
        SVGOptions = new QWidget();
        SVGOptions->setObjectName("SVGOptions");
        verticalLayout_8 = new QVBoxLayout(SVGOptions);
        verticalLayout_8->setSpacing(6);
        verticalLayout_8->setContentsMargins(11, 11, 11, 11);
        verticalLayout_8->setObjectName("verticalLayout_8");
        horizontalLayout_12 = new QHBoxLayout();
        horizontalLayout_12->setSpacing(6);
        horizontalLayout_12->setObjectName("horizontalLayout_12");
        label_3 = new QLabel(SVGOptions);
        label_3->setObjectName("label_3");

        horizontalLayout_12->addWidget(label_3);

        leSVGWidth = new QLineEdit(SVGOptions);
        leSVGWidth->setObjectName("leSVGWidth");
        QSizePolicy sizePolicy10(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Fixed);
        sizePolicy10.setHorizontalStretch(0);
        sizePolicy10.setVerticalStretch(0);
        sizePolicy10.setHeightForWidth(leSVGWidth->sizePolicy().hasHeightForWidth());
        leSVGWidth->setSizePolicy(sizePolicy10);
        leSVGWidth->setMinimumSize(QSize(50, 0));
        leSVGWidth->setMaximumSize(QSize(50, 16777215));

        horizontalLayout_12->addWidget(leSVGWidth);

        horizontalSpacer = new QSpacerItem(18, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_12->addItem(horizontalSpacer);

        label_12 = new QLabel(SVGOptions);
        label_12->setObjectName("label_12");

        horizontalLayout_12->addWidget(label_12);

        leSVGHeight = new QLineEdit(SVGOptions);
        leSVGHeight->setObjectName("leSVGHeight");
        leSVGHeight->setMaximumSize(QSize(50, 16777215));

        horizontalLayout_12->addWidget(leSVGHeight);


        verticalLayout_8->addLayout(horizontalLayout_12);

        line_3 = new QFrame(SVGOptions);
        line_3->setObjectName("line_3");
        line_3->setFrameShape(QFrame::Shape::HLine);
        line_3->setFrameShadow(QFrame::Shadow::Sunken);

        verticalLayout_8->addWidget(line_3);

        cbSVGEmbedStyle = new QCheckBox(SVGOptions);
        cbSVGEmbedStyle->setObjectName("cbSVGEmbedStyle");

        verticalLayout_8->addWidget(cbSVGEmbedStyle);

        horizontalLayout_26 = new QHBoxLayout();
        horizontalLayout_26->setSpacing(6);
        horizontalLayout_26->setObjectName("horizontalLayout_26");
        label_29 = new QLabel(SVGOptions);
        label_29->setObjectName("label_29");

        horizontalLayout_26->addWidget(label_29);

        leSVGStyleFile = new QLineEdit(SVGOptions);
        leSVGStyleFile->setObjectName("leSVGStyleFile");
        leSVGStyleFile->setMaximumSize(QSize(100, 16777215));

        horizontalLayout_26->addWidget(leSVGStyleFile);


        verticalLayout_8->addLayout(horizontalLayout_26);

        horizontalLayout_27 = new QHBoxLayout();
        horizontalLayout_27->setSpacing(6);
        horizontalLayout_27->setObjectName("horizontalLayout_27");
        label_30 = new QLabel(SVGOptions);
        label_30->setObjectName("label_30");

        horizontalLayout_27->addWidget(label_30);

        horizontalSpacer_6 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_27->addItem(horizontalSpacer_6);


        verticalLayout_8->addLayout(horizontalLayout_27);

        horizontalLayout_28 = new QHBoxLayout();
        horizontalLayout_28->setSpacing(6);
        horizontalLayout_28->setObjectName("horizontalLayout_28");
        leSVGStyleIncFile = new QLineEdit(SVGOptions);
        leSVGStyleIncFile->setObjectName("leSVGStyleIncFile");

        horizontalLayout_28->addWidget(leSVGStyleIncFile);

        pbSVGChooseStyleIncFile = new QPushButton(SVGOptions);
        pbSVGChooseStyleIncFile->setObjectName("pbSVGChooseStyleIncFile");
        pbSVGChooseStyleIncFile->setMinimumSize(QSize(20, 0));

        horizontalLayout_28->addWidget(pbSVGChooseStyleIncFile);


        verticalLayout_8->addLayout(horizontalLayout_28);

        verticalSpacer_10 = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_8->addItem(verticalSpacer_10);

        stackedSpecificOptions->addWidget(SVGOptions);
        EmptyPage = new QWidget();
        EmptyPage->setObjectName("EmptyPage");
        verticalLayout_11 = new QVBoxLayout(EmptyPage);
        verticalLayout_11->setSpacing(6);
        verticalLayout_11->setContentsMargins(11, 11, 11, 11);
        verticalLayout_11->setObjectName("verticalLayout_11");
        label_7 = new QLabel(EmptyPage);
        label_7->setObjectName("label_7");

        verticalLayout_11->addWidget(label_7);

        verticalSpacer_3 = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_11->addItem(verticalSpacer_3);

        stackedSpecificOptions->addWidget(EmptyPage);

        gridLayout->addWidget(stackedSpecificOptions, 0, 0, 1, 1);

        tabWidgetOptions->addTab(page_output_spec, QString());
        page_formatting = new QWidget();
        page_formatting->setObjectName("page_formatting");
        verticalLayout_6 = new QVBoxLayout(page_formatting);
        verticalLayout_6->setSpacing(6);
        verticalLayout_6->setContentsMargins(11, 11, 11, 11);
        verticalLayout_6->setObjectName("verticalLayout_6");
        horizontalLayout_3 = new QHBoxLayout();
        horizontalLayout_3->setSpacing(6);
        horizontalLayout_3->setObjectName("horizontalLayout_3");
        label_5 = new QLabel(page_formatting);
        label_5->setObjectName("label_5");
        sizePolicy7.setHeightForWidth(label_5->sizePolicy().hasHeightForWidth());
        label_5->setSizePolicy(sizePolicy7);
        label_5->setMinimumSize(QSize(110, 0));
        label_5->setMaximumSize(QSize(110, 16777215));

        horizontalLayout_3->addWidget(label_5);

        horizontalSpacer_7 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_3->addItem(horizontalSpacer_7);

        comboThemeFilter = new QComboBox(page_formatting);
        comboThemeFilter->setObjectName("comboThemeFilter");

        horizontalLayout_3->addWidget(comboThemeFilter);

        comboTheme = new QComboBox(page_formatting);
        comboTheme->setObjectName("comboTheme");
        QSizePolicy sizePolicy11(QSizePolicy::Policy::MinimumExpanding, QSizePolicy::Policy::Fixed);
        sizePolicy11.setHorizontalStretch(0);
        sizePolicy11.setVerticalStretch(0);
        sizePolicy11.setHeightForWidth(comboTheme->sizePolicy().hasHeightForWidth());
        comboTheme->setSizePolicy(sizePolicy11);
        comboTheme->setMinimumSize(QSize(120, 0));
        comboTheme->setMaximumSize(QSize(200, 16777215));
        comboTheme->setInsertPolicy(QComboBox::InsertAlphabetically);
        comboTheme->setSizeAdjustPolicy(QComboBox::AdjustToContents);

        horizontalLayout_3->addWidget(comboTheme);


        verticalLayout_6->addLayout(horizontalLayout_3);

        horizontalLayout_4 = new QHBoxLayout();
        horizontalLayout_4->setSpacing(6);
        horizontalLayout_4->setObjectName("horizontalLayout_4");
        cbReformat = new QCheckBox(page_formatting);
        cbReformat->setObjectName("cbReformat");

        horizontalLayout_4->addWidget(cbReformat);

        comboReformat = new QComboBox(page_formatting);
        comboReformat->setObjectName("comboReformat");
        sizePolicy8.setHeightForWidth(comboReformat->sizePolicy().hasHeightForWidth());
        comboReformat->setSizePolicy(sizePolicy8);
        comboReformat->setMaximumSize(QSize(120, 16777215));
        comboReformat->setMaxVisibleItems(10);

        horizontalLayout_4->addWidget(comboReformat);


        verticalLayout_6->addLayout(horizontalLayout_4);

        horizontalLayout_5 = new QHBoxLayout();
        horizontalLayout_5->setSpacing(6);
        horizontalLayout_5->setObjectName("horizontalLayout_5");
        cbKwCase = new QCheckBox(page_formatting);
        cbKwCase->setObjectName("cbKwCase");

        horizontalLayout_5->addWidget(cbKwCase);

        comboKwCase = new QComboBox(page_formatting);
        comboKwCase->addItem(QString());
        comboKwCase->addItem(QString());
        comboKwCase->addItem(QString());
        comboKwCase->setObjectName("comboKwCase");
        sizePolicy8.setHeightForWidth(comboKwCase->sizePolicy().hasHeightForWidth());
        comboKwCase->setSizePolicy(sizePolicy8);
        comboKwCase->setMaximumSize(QSize(120, 16777215));
        comboKwCase->setSizeAdjustPolicy(QComboBox::AdjustToContents);

        horizontalLayout_5->addWidget(comboKwCase);


        verticalLayout_6->addLayout(horizontalLayout_5);

        horizontalLayout_6 = new QHBoxLayout();
        horizontalLayout_6->setSpacing(6);
        horizontalLayout_6->setObjectName("horizontalLayout_6");
        label_6 = new QLabel(page_formatting);
        label_6->setObjectName("label_6");
        sizePolicy8.setHeightForWidth(label_6->sizePolicy().hasHeightForWidth());
        label_6->setSizePolicy(sizePolicy8);

        horizontalLayout_6->addWidget(label_6);

        sbTabWidth = new QSpinBox(page_formatting);
        sbTabWidth->setObjectName("sbTabWidth");
        sbTabWidth->setMaximumSize(QSize(120, 16777215));
        sbTabWidth->setMaximum(10);

        horizontalLayout_6->addWidget(sbTabWidth);


        verticalLayout_6->addLayout(horizontalLayout_6);

        gridLayout_2 = new QGridLayout();
        gridLayout_2->setSpacing(6);
        gridLayout_2->setObjectName("gridLayout_2");
        cbWrapping = new QCheckBox(page_formatting);
        cbWrapping->setObjectName("cbWrapping");

        gridLayout_2->addWidget(cbWrapping, 0, 0, 1, 1);

        sbLineLength = new QSpinBox(page_formatting);
        sbLineLength->setObjectName("sbLineLength");
        sbLineLength->setMaximumSize(QSize(60, 16777215));
        sbLineLength->setMinimum(60);
        sbLineLength->setMaximum(120);
        sbLineLength->setSingleStep(2);

        gridLayout_2->addWidget(sbLineLength, 0, 1, 1, 1);

        cbAdvWrapping = new QCheckBox(page_formatting);
        cbAdvWrapping->setObjectName("cbAdvWrapping");
        cbAdvWrapping->setEnabled(false);

        gridLayout_2->addWidget(cbAdvWrapping, 1, 0, 1, 2);

        cbOmitWrappedLineNumbers = new QCheckBox(page_formatting);
        cbOmitWrappedLineNumbers->setObjectName("cbOmitWrappedLineNumbers");

        gridLayout_2->addWidget(cbOmitWrappedLineNumbers, 2, 0, 1, 2);


        verticalLayout_6->addLayout(gridLayout_2);

        gridLayout_9 = new QGridLayout();
        gridLayout_9->setSpacing(6);
        gridLayout_9->setObjectName("gridLayout_9");
        label_23 = new QLabel(page_formatting);
        label_23->setObjectName("label_23");

        gridLayout_9->addWidget(label_23, 0, 0, 1, 1);

        comboFontName = new QFontComboBox(page_formatting);
        comboFontName->setObjectName("comboFontName");
        comboFontName->setMaximumSize(QSize(150, 16777215));
        QFont font3;
        comboFontName->setFont(font3);
        comboFontName->setFontFilters(QFontComboBox::MonospacedFonts);
        QFont font4;
        font4.setFamilies({QString::fromUtf8("Consolas")});
        comboFontName->setCurrentFont(font4);

        gridLayout_9->addWidget(comboFontName, 0, 1, 1, 1);

        label_4 = new QLabel(page_formatting);
        label_4->setObjectName("label_4");

        gridLayout_9->addWidget(label_4, 1, 0, 1, 1);

        leFontSize = new QLineEdit(page_formatting);
        leFontSize->setObjectName("leFontSize");
        sizePolicy5.setHeightForWidth(leFontSize->sizePolicy().hasHeightForWidth());
        leFontSize->setSizePolicy(sizePolicy5);
        leFontSize->setMaximumSize(QSize(150, 16777215));

        gridLayout_9->addWidget(leFontSize, 1, 1, 1, 1);


        verticalLayout_6->addLayout(gridLayout_9);

        tabWidgetOptions->addTab(page_formatting, QString());

        verticalLayout_16->addWidget(tabWidgetOptions);

        dockWidget->setWidget(dockWidgetContents);
        MainWindowClass->addDockWidget(Qt::DockWidgetArea::LeftDockWidgetArea, dockWidget);
#if QT_CONFIG(shortcut)
        label_2->setBuddy(comboFormat);
        label_21->setBuddy(leLATEXStyleFile);
        label_24->setBuddy(leTEXStyleFile);
        label_3->setBuddy(leSVGWidth);
        label_12->setBuddy(leSVGHeight);
        label_29->setBuddy(leSVGStyleFile);
        label_5->setBuddy(comboTheme);
        label_6->setBuddy(sbTabWidth);
        label_23->setBuddy(comboFontName);
        label_4->setBuddy(leFontSize);
#endif // QT_CONFIG(shortcut)
        QWidget::setTabOrder(pbOpenFiles, lvInputFiles);
        QWidget::setTabOrder(lvInputFiles, pbClearSelection);
        QWidget::setTabOrder(pbClearSelection, pbClearAll);
        QWidget::setTabOrder(pbClearAll, tabWidgetOptions);
        QWidget::setTabOrder(tabWidgetOptions, comboFormat);
        QWidget::setTabOrder(comboFormat, cbIncLineNo);
        QWidget::setTabOrder(cbIncLineNo, sbLineNoWidth);
        QWidget::setTabOrder(sbLineNoWidth, cbPadZeroes);
        QWidget::setTabOrder(cbPadZeroes, cbFragment);
        QWidget::setTabOrder(cbFragment, cbValidateInput);
        QWidget::setTabOrder(cbValidateInput, cbEncoding);
        QWidget::setTabOrder(cbEncoding, comboEncoding);
        QWidget::setTabOrder(comboEncoding, cbHTMLEmbedStyle);
        QWidget::setTabOrder(cbHTMLEmbedStyle, cbHTMLInlineCSS);
        QWidget::setTabOrder(cbHTMLInlineCSS, leHTMLStyleFile);
        QWidget::setTabOrder(leHTMLStyleFile, leHTMLStyleIncFile);
        QWidget::setTabOrder(leHTMLStyleIncFile, pbHTMLChooseStyleIncFile);
        QWidget::setTabOrder(pbHTMLChooseStyleIncFile, leHTMLCssPrefix);
        QWidget::setTabOrder(leHTMLCssPrefix, cbHTMLAnchors);
        QWidget::setTabOrder(cbHTMLAnchors, cbHTMLFileNameAnchor);
        QWidget::setTabOrder(cbHTMLFileNameAnchor, cbHTMLOrderedList);
        QWidget::setTabOrder(cbHTMLOrderedList, cbHTMLEnclosePreTags);
        QWidget::setTabOrder(cbHTMLEnclosePreTags, cbLATEXEscQuotes);
        QWidget::setTabOrder(cbLATEXEscQuotes, cbLATEXBabel);
        QWidget::setTabOrder(cbLATEXBabel, cbLATEXPrettySymbols);
        QWidget::setTabOrder(cbLATEXPrettySymbols, cbLATEXEmbedStyle);
        QWidget::setTabOrder(cbLATEXEmbedStyle, leLATEXStyleFile);
        QWidget::setTabOrder(leLATEXStyleFile, leLATEXStyleIncFile);
        QWidget::setTabOrder(leLATEXStyleIncFile, pbLATEXChooseStyleIncFile);
        QWidget::setTabOrder(pbLATEXChooseStyleIncFile, cbTEXEmbedStyle);
        QWidget::setTabOrder(cbTEXEmbedStyle, leTEXStyleFile);
        QWidget::setTabOrder(leTEXStyleFile, leTEXStyleIncFile);
        QWidget::setTabOrder(leTEXStyleIncFile, pbTEXChooseStyleIncFile);
        QWidget::setTabOrder(pbTEXChooseStyleIncFile, cbRTFCharStyles);
        QWidget::setTabOrder(cbRTFCharStyles, comboRTFPageSize);
        QWidget::setTabOrder(comboRTFPageSize, leSVGWidth);
        QWidget::setTabOrder(leSVGWidth, leSVGHeight);
        QWidget::setTabOrder(leSVGHeight, cbSVGEmbedStyle);
        QWidget::setTabOrder(cbSVGEmbedStyle, leSVGStyleFile);
        QWidget::setTabOrder(leSVGStyleFile, leSVGStyleIncFile);
        QWidget::setTabOrder(leSVGStyleIncFile, pbSVGChooseStyleIncFile);
        QWidget::setTabOrder(pbSVGChooseStyleIncFile, comboTheme);
        QWidget::setTabOrder(comboTheme, cbReformat);
        QWidget::setTabOrder(cbReformat, comboReformat);
        QWidget::setTabOrder(comboReformat, cbKwCase);
        QWidget::setTabOrder(cbKwCase, comboKwCase);
        QWidget::setTabOrder(comboKwCase, sbTabWidth);
        QWidget::setTabOrder(sbTabWidth, cbWrapping);
        QWidget::setTabOrder(cbWrapping, sbLineLength);
        QWidget::setTabOrder(sbLineLength, cbAdvWrapping);
        QWidget::setTabOrder(cbAdvWrapping, comboFontName);
        QWidget::setTabOrder(comboFontName, leFontSize);
        QWidget::setTabOrder(leFontSize, browserPreview);
        QWidget::setTabOrder(browserPreview, pbStartConversion);
        QWidget::setTabOrder(pbStartConversion, pbCopyFile2CP);

        menuBar->addAction(menu_File->menuAction());
        menuBar->addAction(menu_Help->menuAction());
        menu_File->addAction(action_Open_files);
        menu_File->addSeparator();
        menu_File->addAction(action_Exit);
        menu_Help->addAction(action_Tips);
        menu_Help->addAction(action_Manual);
        menu_Help->addAction(action_Plug_Ins);
        menu_Help->addAction(action_Changelog);
        menu_Help->addAction(action_License);
        menu_Help->addSeparator();
        menu_Help->addAction(actionAbout_translations);
        menu_Help->addSeparator();
        menu_Help->addAction(actionVisit_website);
        menu_Help->addAction(action_About_Highlight);

        retranslateUi(MainWindowClass);
        QObject::connect(pbClearAll, &QPushButton::clicked, lvInputFiles, qOverload<>(&QListWidget::clear));

        tabIOSelection->setCurrentIndex(0);
        pbStartConversion->setDefault(true);
        tabWidgetOptions->setCurrentIndex(2);
        stackedSpecificOptions->setCurrentIndex(0);
        tabWidget->setCurrentIndex(0);
        comboRTFPageSize->setCurrentIndex(1);
        comboTheme->setCurrentIndex(-1);


        QMetaObject::connectSlotsByName(MainWindowClass);
    } // setupUi

    void retranslateUi(QMainWindow *MainWindowClass)
    {
        MainWindowClass->setWindowTitle(QCoreApplication::translate("MainWindowClass", "Highlight", nullptr));
        action_Open_files->setText(QCoreApplication::translate("MainWindowClass", "&Open files", nullptr));
        action_Exit->setText(QCoreApplication::translate("MainWindowClass", "&Exit", nullptr));
        action_Load->setText(QCoreApplication::translate("MainWindowClass", "&Load", nullptr));
        action_Save->setText(QCoreApplication::translate("MainWindowClass", "&Save", nullptr));
        actionLoad_default_project->setText(QCoreApplication::translate("MainWindowClass", "Load &default project", nullptr));
        action_Manual->setText(QCoreApplication::translate("MainWindowClass", "&Readme", nullptr));
        action_Tips->setText(QCoreApplication::translate("MainWindowClass", "&Tips", nullptr));
        action_Changelog->setText(QCoreApplication::translate("MainWindowClass", "&Changelog", nullptr));
        action_License->setText(QCoreApplication::translate("MainWindowClass", "&License", nullptr));
        action_About_Highlight->setText(QCoreApplication::translate("MainWindowClass", "&About Highlight", nullptr));
        actionAbout_translations->setText(QCoreApplication::translate("MainWindowClass", "A&bout translations", nullptr));
        actionVisit_website->setText(QCoreApplication::translate("MainWindowClass", "&Visit andre-simon.de", nullptr));
        actionDock_floating_panels->setText(QCoreApplication::translate("MainWindowClass", "&Dock floating panels", nullptr));
        action_Plug_Ins->setText(QCoreApplication::translate("MainWindowClass", "&Plug-Ins", nullptr));
        action_File_access_trace_W32->setText(QCoreApplication::translate("MainWindowClass", "&File access trace (W32)", nullptr));
        lbPreview->setText(QCoreApplication::translate("MainWindowClass", "Preview", nullptr));
#if QT_CONFIG(tooltip)
        browserPreview->setToolTip(QString());
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(whatsthis)
        browserPreview->setWhatsThis(QString());
#endif // QT_CONFIG(whatsthis)
        label_14->setText(QCoreApplication::translate("MainWindowClass", "Output progress:", nullptr));
        menu_File->setTitle(QCoreApplication::translate("MainWindowClass", "&File", nullptr));
        menu_Help->setTitle(QCoreApplication::translate("MainWindowClass", "&Help", nullptr));
        dockWidget->setWindowTitle(QCoreApplication::translate("MainWindowClass", "Highlighting options", nullptr));
#if QT_CONFIG(tooltip)
        pbOpenFiles->setToolTip(QCoreApplication::translate("MainWindowClass", "Choose the source code files you want to convert.", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(whatsthis)
        pbOpenFiles->setWhatsThis(QString());
#endif // QT_CONFIG(whatsthis)
        pbOpenFiles->setText(QCoreApplication::translate("MainWindowClass", "Choose input files", nullptr));
#if QT_CONFIG(tooltip)
        lvInputFiles->setToolTip(QCoreApplication::translate("MainWindowClass", "List of input files.", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        pbClearSelection->setToolTip(QCoreApplication::translate("MainWindowClass", "Remove the selected input files.", nullptr));
#endif // QT_CONFIG(tooltip)
        pbClearSelection->setText(QCoreApplication::translate("MainWindowClass", "Clear selection", nullptr));
#if QT_CONFIG(tooltip)
        pbClearAll->setToolTip(QCoreApplication::translate("MainWindowClass", "Remove all input files.", nullptr));
#endif // QT_CONFIG(tooltip)
        pbClearAll->setText(QCoreApplication::translate("MainWindowClass", "Clear all", nullptr));
        label_11->setText(QCoreApplication::translate("MainWindowClass", "Output destination:", nullptr));
#if QT_CONFIG(tooltip)
        leOutputDest->setToolTip(QCoreApplication::translate("MainWindowClass", "Output directory", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        pbOutputDest->setToolTip(QCoreApplication::translate("MainWindowClass", "Select the output directory.", nullptr));
#endif // QT_CONFIG(tooltip)
        pbOutputDest->setText(QCoreApplication::translate("MainWindowClass", "...", nullptr));
#if QT_CONFIG(tooltip)
        pbBrowseOutDir->setToolTip(QCoreApplication::translate("MainWindowClass", "Browse output directory", nullptr));
#endif // QT_CONFIG(tooltip)
        pbBrowseOutDir->setText(QString());
#if QT_CONFIG(tooltip)
        cbWrite2Src->setToolTip(QCoreApplication::translate("MainWindowClass", "Save output in the input file directories.", nullptr));
#endif // QT_CONFIG(tooltip)
        cbWrite2Src->setText(QCoreApplication::translate("MainWindowClass", "Write to source directories", nullptr));
#if QT_CONFIG(tooltip)
        pbStartConversion->setToolTip(QCoreApplication::translate("MainWindowClass", "Start the conversion of your input files.", nullptr));
#endif // QT_CONFIG(tooltip)
        pbStartConversion->setText(QCoreApplication::translate("MainWindowClass", "Convert files", nullptr));
#if QT_CONFIG(tooltip)
        pbCopyFile2CP->setToolTip(QCoreApplication::translate("MainWindowClass", "Copy highlighted code of the selected file into the clipboard.", nullptr));
#endif // QT_CONFIG(tooltip)
        pbCopyFile2CP->setText(QCoreApplication::translate("MainWindowClass", "Copy file to clipboard", nullptr));
        tabIOSelection->setTabText(tabIOSelection->indexOf(tab_file_io), QCoreApplication::translate("MainWindowClass", "Files", nullptr));
        label_13->setText(QCoreApplication::translate("MainWindowClass", "Select syntax:", nullptr));
#if QT_CONFIG(tooltip)
        comboSelectSyntax->setToolTip(QCoreApplication::translate("MainWindowClass", "Select the correct syntax of the code snippet.", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        pbPasteFromCB->setToolTip(QCoreApplication::translate("MainWindowClass", "Paste clipboard content into the preview window.", nullptr));
#endif // QT_CONFIG(tooltip)
        pbPasteFromCB->setText(QCoreApplication::translate("MainWindowClass", "Paste from clipboard (%1)", nullptr));
#if QT_CONFIG(tooltip)
        pbCopyToCP->setToolTip(QCoreApplication::translate("MainWindowClass", "Copy highlighted code into the clipboard.", nullptr));
#endif // QT_CONFIG(tooltip)
        pbCopyToCP->setText(QCoreApplication::translate("MainWindowClass", "Copy preview to clipboard (%1)", nullptr));
#if QT_CONFIG(tooltip)
        pbCopyAndPaste->setToolTip(QCoreApplication::translate("MainWindowClass", "Paste code from clipboard and copy the output back in one step", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(statustip)
        pbCopyAndPaste->setStatusTip(QString());
#endif // QT_CONFIG(statustip)
        pbCopyAndPaste->setText(QCoreApplication::translate("MainWindowClass", "Paste, Convert and Copy", nullptr));
#if QT_CONFIG(tooltip)
        cbCopyRange->setToolTip(QCoreApplication::translate("MainWindowClass", "Only output the selected lines of the preview.", nullptr));
#endif // QT_CONFIG(tooltip)
        cbCopyRange->setText(QCoreApplication::translate("MainWindowClass", "Output selected lines only", nullptr));
        tabIOSelection->setTabText(tabIOSelection->indexOf(tab_clipboard), QCoreApplication::translate("MainWindowClass", "Clipboard", nullptr));
#if QT_CONFIG(tooltip)
        pbSelectPlugin->setToolTip(QCoreApplication::translate("MainWindowClass", "Choose a plug-in script.", nullptr));
#endif // QT_CONFIG(tooltip)
        pbSelectPlugin->setText(QCoreApplication::translate("MainWindowClass", "Add plug-in to pool", nullptr));
#if QT_CONFIG(tooltip)
        lvPluginScripts->setToolTip(QCoreApplication::translate("MainWindowClass", "List of plug-ins. Toggle checkbox to enable the scripts. The preview window may not display all plug-in effects.", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        pbClearSelPlugin->setToolTip(QCoreApplication::translate("MainWindowClass", "Remove the selected plug-ins.", nullptr));
#endif // QT_CONFIG(tooltip)
        pbClearSelPlugin->setText(QCoreApplication::translate("MainWindowClass", "Clear selection", nullptr));
#if QT_CONFIG(tooltip)
        pbClearAllPlugins->setToolTip(QCoreApplication::translate("MainWindowClass", "Remove all plug-ins.", nullptr));
#endif // QT_CONFIG(tooltip)
        pbClearAllPlugins->setText(QCoreApplication::translate("MainWindowClass", "Clear all", nullptr));
        groupPluginDescription->setTitle(QCoreApplication::translate("MainWindowClass", "Description", nullptr));
#if QT_CONFIG(tooltip)
        lblPluginDescription->setToolTip(QCoreApplication::translate("MainWindowClass", "Shows description of a selected plug-in script.", nullptr));
#endif // QT_CONFIG(tooltip)
        lblPluginDescription->setText(QCoreApplication::translate("MainWindowClass", "Plug-In description", nullptr));
        label_15->setText(QCoreApplication::translate("MainWindowClass", "Plug-in parameter", nullptr));
#if QT_CONFIG(tooltip)
        lePluginReadFilePath->setToolTip(QCoreApplication::translate("MainWindowClass", "Optional plug-in parameter, this may be a path to a plug-in input file", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        pbPluginReadFilePath->setToolTip(QCoreApplication::translate("MainWindowClass", "Select the plug-in input file.", nullptr));
#endif // QT_CONFIG(tooltip)
        pbPluginReadFilePath->setText(QCoreApplication::translate("MainWindowClass", "...", nullptr));
        tabIOSelection->setTabText(tabIOSelection->indexOf(tab_plugins), QCoreApplication::translate("MainWindowClass", "Plug-ins", nullptr));
#if QT_CONFIG(tooltip)
        pbSelectScript->setToolTip(QCoreApplication::translate("MainWindowClass", "Choose a custom syntax or theme script.", nullptr));
#endif // QT_CONFIG(tooltip)
        pbSelectScript->setText(QCoreApplication::translate("MainWindowClass", "Add custom script to pool", nullptr));
#if QT_CONFIG(tooltip)
        lvUserScripts->setToolTip(QCoreApplication::translate("MainWindowClass", "You can check one file of each type (syntax and theme) to override the default behaviour. If checked, the scripts will be watched for changes.", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        pbClearSelScript->setToolTip(QCoreApplication::translate("MainWindowClass", "Remove the selected scripts.", nullptr));
#endif // QT_CONFIG(tooltip)
        pbClearSelScript->setText(QCoreApplication::translate("MainWindowClass", "Clear selection", nullptr));
#if QT_CONFIG(tooltip)
        pbClearAllScripts->setToolTip(QCoreApplication::translate("MainWindowClass", "Remove all scripts.", nullptr));
#endif // QT_CONFIG(tooltip)
        pbClearAllScripts->setText(QCoreApplication::translate("MainWindowClass", "Clear all", nullptr));
        groupScriptDescription->setTitle(QCoreApplication::translate("MainWindowClass", "Description", nullptr));
#if QT_CONFIG(tooltip)
        lblScriptDescription->setToolTip(QCoreApplication::translate("MainWindowClass", "Shows description of a selected script.", nullptr));
#endif // QT_CONFIG(tooltip)
        lblScriptDescription->setText(QCoreApplication::translate("MainWindowClass", "Script description", nullptr));
        tabIOSelection->setTabText(tabIOSelection->indexOf(tab_scripts), QCoreApplication::translate("MainWindowClass", "Scripts", nullptr));
#if QT_CONFIG(tooltip)
        tab_lsp->setToolTip(QCoreApplication::translate("MainWindowClass", "Configure the LSP Client", nullptr));
#endif // QT_CONFIG(tooltip)
        label_17->setText(QCoreApplication::translate("MainWindowClass", "Executable:", nullptr));
#if QT_CONFIG(tooltip)
        leLSExec->setToolTip(QCoreApplication::translate("MainWindowClass", "Path of the server executable.", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        pbSelExecutable->setToolTip(QCoreApplication::translate("MainWindowClass", "Select the server executable.", nullptr));
#endif // QT_CONFIG(tooltip)
        pbSelExecutable->setText(QCoreApplication::translate("MainWindowClass", "...", nullptr));
        label_10->setText(QCoreApplication::translate("MainWindowClass", "Language Server:", nullptr));
#if QT_CONFIG(tooltip)
        comboLSProfiles->setToolTip(QCoreApplication::translate("MainWindowClass", "Select a Language Server profile.", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        cbLSDebug->setToolTip(QCoreApplication::translate("MainWindowClass", "Print LSP messages to standard error", nullptr));
#endif // QT_CONFIG(tooltip)
        cbLSDebug->setText(QCoreApplication::translate("MainWindowClass", "Output Logs", nullptr));
#if QT_CONFIG(tooltip)
        pbLSInitialize->setToolTip(QCoreApplication::translate("MainWindowClass", "Start an initialization request.", nullptr));
#endif // QT_CONFIG(tooltip)
        pbLSInitialize->setText(QCoreApplication::translate("MainWindowClass", "Check Capabilities", nullptr));
        label_16->setText(QCoreApplication::translate("MainWindowClass", "Workspace:", nullptr));
#if QT_CONFIG(tooltip)
        leLSWorkspace->setToolTip(QCoreApplication::translate("MainWindowClass", "The workspace directory.", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        pbSelWorkspace->setToolTip(QCoreApplication::translate("MainWindowClass", "Select the workspace directory of your project.", nullptr));
#endif // QT_CONFIG(tooltip)
        pbSelWorkspace->setText(QCoreApplication::translate("MainWindowClass", "...", nullptr));
#if QT_CONFIG(tooltip)
        cbLSSemantic->setToolTip(QCoreApplication::translate("MainWindowClass", "Improve token recognition with semantic highlighting.", nullptr));
#endif // QT_CONFIG(tooltip)
        cbLSSemantic->setText(QCoreApplication::translate("MainWindowClass", "Semantic Highlighting", nullptr));
#if QT_CONFIG(tooltip)
        cbLSSyntaxErrors->setToolTip(QCoreApplication::translate("MainWindowClass", "Show syntax errors and messages (depends on output format).", nullptr));
#endif // QT_CONFIG(tooltip)
        cbLSSyntaxErrors->setText(QCoreApplication::translate("MainWindowClass", "Show Syntax Errors", nullptr));
#if QT_CONFIG(tooltip)
        cbLSHover->setToolTip(QCoreApplication::translate("MainWindowClass", "Run hover requests to add tooltips to your output (HTML only).", nullptr));
#endif // QT_CONFIG(tooltip)
        cbLSHover->setText(QCoreApplication::translate("MainWindowClass", "Hover", nullptr));
#if QT_CONFIG(tooltip)
        cbLSLegacy->setToolTip(QCoreApplication::translate("MainWindowClass", "Do not require a server capabilities response", nullptr));
#endif // QT_CONFIG(tooltip)
        cbLSLegacy->setText(QCoreApplication::translate("MainWindowClass", "Legacy Protocol", nullptr));
        tabIOSelection->setTabText(tabIOSelection->indexOf(tab_lsp), QCoreApplication::translate("MainWindowClass", "LSP", nullptr));
        label_2->setText(QCoreApplication::translate("MainWindowClass", "Output for&mat:", nullptr));
        comboFormat->setItemText(0, QCoreApplication::translate("MainWindowClass", "HTML", nullptr));
        comboFormat->setItemText(1, QCoreApplication::translate("MainWindowClass", "XHTML", nullptr));
        comboFormat->setItemText(2, QCoreApplication::translate("MainWindowClass", "LaTeX", nullptr));
        comboFormat->setItemText(3, QCoreApplication::translate("MainWindowClass", "TeX", nullptr));
        comboFormat->setItemText(4, QCoreApplication::translate("MainWindowClass", "RTF", nullptr));
        comboFormat->setItemText(5, QCoreApplication::translate("MainWindowClass", "ODT", nullptr));
        comboFormat->setItemText(6, QCoreApplication::translate("MainWindowClass", "SVG", nullptr));
        comboFormat->setItemText(7, QCoreApplication::translate("MainWindowClass", "BBCode", nullptr));
        comboFormat->setItemText(8, QCoreApplication::translate("MainWindowClass", "ANSI ESC", nullptr));
        comboFormat->setItemText(9, QCoreApplication::translate("MainWindowClass", "XTerm256 ESC", nullptr));
        comboFormat->setItemText(10, QCoreApplication::translate("MainWindowClass", "Truecolor ESC", nullptr));

#if QT_CONFIG(tooltip)
        comboFormat->setToolTip(QCoreApplication::translate("MainWindowClass", "Choose an output format.", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        cbIncLineNo->setToolTip(QCoreApplication::translate("MainWindowClass", "Add line numbers to the output.", nullptr));
#endif // QT_CONFIG(tooltip)
        cbIncLineNo->setText(QCoreApplication::translate("MainWindowClass", "Add line numbers", nullptr));
#if QT_CONFIG(tooltip)
        sbLineNoStart->setToolTip(QCoreApplication::translate("MainWindowClass", "Set line numbering start.", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        sbLineNoWidth->setToolTip(QCoreApplication::translate("MainWindowClass", "Select the line number width.", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        cbPadZeroes->setToolTip(QCoreApplication::translate("MainWindowClass", "Fill leading space of line numbers with zeroes.", nullptr));
#endif // QT_CONFIG(tooltip)
        cbPadZeroes->setText(QCoreApplication::translate("MainWindowClass", "Pad with zeroes", nullptr));
#if QT_CONFIG(tooltip)
        cbFragment->setToolTip(QCoreApplication::translate("MainWindowClass", "Generate output without document header and footer.", nullptr));
#endif // QT_CONFIG(tooltip)
        cbFragment->setText(QCoreApplication::translate("MainWindowClass", "Omit header and footer", nullptr));
#if QT_CONFIG(tooltip)
        cbKeepInjections->setToolTip(QCoreApplication::translate("MainWindowClass", "Output any plug-in text injections even if document header and footer is omitted.", nullptr));
#endif // QT_CONFIG(tooltip)
        cbKeepInjections->setText(QCoreApplication::translate("MainWindowClass", "Keep Plug-In injections", nullptr));
#if QT_CONFIG(tooltip)
        cbValidateInput->setToolTip(QCoreApplication::translate("MainWindowClass", "Test if input data is not binary.\n"
"Removes Unicode BOM mark.", nullptr));
#endif // QT_CONFIG(tooltip)
        cbValidateInput->setText(QCoreApplication::translate("MainWindowClass", "Validate input data", nullptr));
#if QT_CONFIG(tooltip)
        cbOmitVersionInfo->setToolTip(QCoreApplication::translate("MainWindowClass", "Generate output without version information comment.", nullptr));
#endif // QT_CONFIG(tooltip)
        cbOmitVersionInfo->setText(QCoreApplication::translate("MainWindowClass", "Omit version info comment", nullptr));
#if QT_CONFIG(tooltip)
        cbEncoding->setToolTip(QCoreApplication::translate("MainWindowClass", "Set the output file encoding.", nullptr));
#endif // QT_CONFIG(tooltip)
        cbEncoding->setText(QCoreApplication::translate("MainWindowClass", "Set encoding:", nullptr));
        comboEncoding->setItemText(0, QCoreApplication::translate("MainWindowClass", "ISO-8859-1", nullptr));
        comboEncoding->setItemText(1, QCoreApplication::translate("MainWindowClass", "ISO-8859-2", nullptr));
        comboEncoding->setItemText(2, QCoreApplication::translate("MainWindowClass", "ISO-8859-3", nullptr));
        comboEncoding->setItemText(3, QCoreApplication::translate("MainWindowClass", "ISO-8859-4", nullptr));
        comboEncoding->setItemText(4, QCoreApplication::translate("MainWindowClass", "ISO-8859-5", nullptr));
        comboEncoding->setItemText(5, QCoreApplication::translate("MainWindowClass", "ISO-8859-6", nullptr));
        comboEncoding->setItemText(6, QCoreApplication::translate("MainWindowClass", "ISO-8859-7", nullptr));
        comboEncoding->setItemText(7, QCoreApplication::translate("MainWindowClass", "ISO-8859-8", nullptr));
        comboEncoding->setItemText(8, QCoreApplication::translate("MainWindowClass", "ISO-8859-9", nullptr));
        comboEncoding->setItemText(9, QCoreApplication::translate("MainWindowClass", "ISO-8859-10", nullptr));
        comboEncoding->setItemText(10, QCoreApplication::translate("MainWindowClass", "ISO-8859-11", nullptr));
        comboEncoding->setItemText(11, QCoreApplication::translate("MainWindowClass", "ISO-8859-12", nullptr));
        comboEncoding->setItemText(12, QCoreApplication::translate("MainWindowClass", "ISO-8859-13", nullptr));
        comboEncoding->setItemText(13, QCoreApplication::translate("MainWindowClass", "ISO-8859-14", nullptr));
        comboEncoding->setItemText(14, QCoreApplication::translate("MainWindowClass", "ISO-8859-15", nullptr));
        comboEncoding->setItemText(15, QCoreApplication::translate("MainWindowClass", "UTF-8", nullptr));

#if QT_CONFIG(tooltip)
        comboEncoding->setToolTip(QCoreApplication::translate("MainWindowClass", "Select or define the encoding.\n"
"The result has to match the input file encoding.", nullptr));
#endif // QT_CONFIG(tooltip)
        tabWidgetOptions->setTabText(tabWidgetOptions->indexOf(page_general), QCoreApplication::translate("MainWindowClass", "General", nullptr));
#if QT_CONFIG(tooltip)
        cbHTMLEmbedStyle->setToolTip(QCoreApplication::translate("MainWindowClass", "Include the style information in each output file.", nullptr));
#endif // QT_CONFIG(tooltip)
        cbHTMLEmbedStyle->setText(QCoreApplication::translate("MainWindowClass", "Embed style (CSS)", nullptr));
#if QT_CONFIG(tooltip)
        cbHTMLInlineCSS->setToolTip(QCoreApplication::translate("MainWindowClass", "Add CSS information to each tag (do not use CSS class definitions).", nullptr));
#endif // QT_CONFIG(tooltip)
        cbHTMLInlineCSS->setText(QCoreApplication::translate("MainWindowClass", "Inline CSS", nullptr));
        label->setText(QCoreApplication::translate("MainWindowClass", "CSS class prefix:", nullptr));
#if QT_CONFIG(tooltip)
        leHTMLCssPrefix->setToolTip(QCoreApplication::translate("MainWindowClass", "Add a CSS class name prefix to avoid namespace clashes.", nullptr));
#endif // QT_CONFIG(tooltip)
        leHTMLCssPrefix->setText(QString());
        label_19->setText(QCoreApplication::translate("MainWindowClass", "Stylesheet file:", nullptr));
#if QT_CONFIG(tooltip)
        leHTMLStyleFile->setToolTip(QCoreApplication::translate("MainWindowClass", "Name of the referenced style file.", nullptr));
#endif // QT_CONFIG(tooltip)
        leHTMLStyleFile->setText(QCoreApplication::translate("MainWindowClass", "highlight.css", nullptr));
        label_20->setText(QCoreApplication::translate("MainWindowClass", "Stylesheet include file:", nullptr));
#if QT_CONFIG(tooltip)
        leHTMLStyleIncFile->setToolTip(QCoreApplication::translate("MainWindowClass", "Path of the CSS include file.", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        pbHTMLChooseStyleIncFile->setToolTip(QCoreApplication::translate("MainWindowClass", "Select a CSS include file.", nullptr));
#endif // QT_CONFIG(tooltip)
        pbHTMLChooseStyleIncFile->setText(QCoreApplication::translate("MainWindowClass", "...", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(tabWidgetPage1), QCoreApplication::translate("MainWindowClass", "Stylesheets", nullptr));
        label_8->setText(QCoreApplication::translate("MainWindowClass", "Line numbering options:", nullptr));
#if QT_CONFIG(tooltip)
        cbHTMLAnchors->setToolTip(QCoreApplication::translate("MainWindowClass", "Add an anchor to each line.", nullptr));
#endif // QT_CONFIG(tooltip)
        cbHTMLAnchors->setText(QCoreApplication::translate("MainWindowClass", "Add line anchors", nullptr));
#if QT_CONFIG(tooltip)
        cbHTMLFileNameAnchor->setToolTip(QCoreApplication::translate("MainWindowClass", "Add the filename as prefix to the anchors.", nullptr));
#endif // QT_CONFIG(tooltip)
        cbHTMLFileNameAnchor->setText(QCoreApplication::translate("MainWindowClass", "Include file name in anchor", nullptr));
#if QT_CONFIG(tooltip)
        cbHTMLOrderedList->setToolTip(QCoreApplication::translate("MainWindowClass", "Output the lines within an ordered list.", nullptr));
#endif // QT_CONFIG(tooltip)
        cbHTMLOrderedList->setText(QCoreApplication::translate("MainWindowClass", "Output as ordered list", nullptr));
#if QT_CONFIG(tooltip)
        cbHTMLIndex->setToolTip(QCoreApplication::translate("MainWindowClass", "Generate an index file with hyperlinks to all outputted files.", nullptr));
#endif // QT_CONFIG(tooltip)
        cbHTMLIndex->setText(QCoreApplication::translate("MainWindowClass", "Generate index file", nullptr));
#if QT_CONFIG(tooltip)
        cbHTMLEnclosePreTags->setToolTip(QCoreApplication::translate("MainWindowClass", "Add &lt;pre&gt; tags to the output, if the flag \"No document header and footer\" is selected.", nullptr));
#endif // QT_CONFIG(tooltip)
        cbHTMLEnclosePreTags->setText(QCoreApplication::translate("MainWindowClass", "Enclose in pre tags", nullptr));
#if QT_CONFIG(tooltip)
        cbHTMLPasteMIME->setToolTip(QCoreApplication::translate("MainWindowClass", "Add HTML MIME Type when copying code to the clipboard", nullptr));
#endif // QT_CONFIG(tooltip)
        cbHTMLPasteMIME->setText(QCoreApplication::translate("MainWindowClass", "Copy with MIME type", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(tabWidgetPage3), QCoreApplication::translate("MainWindowClass", "Misc", nullptr));
#if QT_CONFIG(tooltip)
        cbLATEXEscQuotes->setToolTip(QCoreApplication::translate("MainWindowClass", "Replace quotes by dq sequences.", nullptr));
#endif // QT_CONFIG(tooltip)
        cbLATEXEscQuotes->setText(QCoreApplication::translate("MainWindowClass", "Escape quotes", nullptr));
#if QT_CONFIG(tooltip)
        cbLATEXBabel->setToolTip(QCoreApplication::translate("MainWindowClass", "Adapt output for the Babel package", nullptr));
#endif // QT_CONFIG(tooltip)
        cbLATEXBabel->setText(QCoreApplication::translate("MainWindowClass", "Add Babel compatibility", nullptr));
#if QT_CONFIG(tooltip)
        cbLATEXBeamer->setToolTip(QCoreApplication::translate("MainWindowClass", "Adapt output for the Beamer package", nullptr));
#endif // QT_CONFIG(tooltip)
        cbLATEXBeamer->setText(QCoreApplication::translate("MainWindowClass", "Add Beamer compatibility", nullptr));
#if QT_CONFIG(tooltip)
        cbLATEXPrettySymbols->setToolTip(QCoreApplication::translate("MainWindowClass", "Replace default symbols (brackets, tilde) by nice redefinitions.", nullptr));
#endif // QT_CONFIG(tooltip)
        cbLATEXPrettySymbols->setText(QCoreApplication::translate("MainWindowClass", "Add pretty symbols", nullptr));
#if QT_CONFIG(tooltip)
        cbLATEXEmbedStyle->setToolTip(QCoreApplication::translate("MainWindowClass", "Include the style information in each output file.", nullptr));
#endif // QT_CONFIG(tooltip)
        cbLATEXEmbedStyle->setText(QCoreApplication::translate("MainWindowClass", "Embed style (defs)", nullptr));
        label_21->setText(QCoreApplication::translate("MainWindowClass", "Stylesheet file:", nullptr));
#if QT_CONFIG(tooltip)
        leLATEXStyleFile->setToolTip(QCoreApplication::translate("MainWindowClass", "Name of the referenced  style file.", nullptr));
#endif // QT_CONFIG(tooltip)
        leLATEXStyleFile->setText(QCoreApplication::translate("MainWindowClass", "highlight.sty", nullptr));
        label_22->setText(QCoreApplication::translate("MainWindowClass", "Stylesheet include file:", nullptr));
#if QT_CONFIG(tooltip)
        leLATEXStyleIncFile->setToolTip(QCoreApplication::translate("MainWindowClass", "Path of the style include file.", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        pbLATEXChooseStyleIncFile->setToolTip(QCoreApplication::translate("MainWindowClass", "Select a style include file.", nullptr));
#endif // QT_CONFIG(tooltip)
        pbLATEXChooseStyleIncFile->setText(QCoreApplication::translate("MainWindowClass", "...", nullptr));
#if QT_CONFIG(tooltip)
        cbTEXEmbedStyle->setToolTip(QCoreApplication::translate("MainWindowClass", "Include the style information in each output file.", nullptr));
#endif // QT_CONFIG(tooltip)
        cbTEXEmbedStyle->setText(QCoreApplication::translate("MainWindowClass", "Embed style (macros)", nullptr));
        label_24->setText(QCoreApplication::translate("MainWindowClass", "Stylesheet file:", nullptr));
#if QT_CONFIG(tooltip)
        leTEXStyleFile->setToolTip(QCoreApplication::translate("MainWindowClass", "Name of the referenced style file.", nullptr));
#endif // QT_CONFIG(tooltip)
        leTEXStyleFile->setText(QCoreApplication::translate("MainWindowClass", "highlight.sty", nullptr));
        label_25->setText(QCoreApplication::translate("MainWindowClass", "Stylesheet include file:", nullptr));
#if QT_CONFIG(tooltip)
        leTEXStyleIncFile->setToolTip(QCoreApplication::translate("MainWindowClass", "Path of the style include file.", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        pbTEXChooseStyleIncFile->setToolTip(QCoreApplication::translate("MainWindowClass", "Select a style include file.", nullptr));
#endif // QT_CONFIG(tooltip)
        pbTEXChooseStyleIncFile->setText(QCoreApplication::translate("MainWindowClass", "...", nullptr));
#if QT_CONFIG(tooltip)
        cbRTFCharStyles->setToolTip(QCoreApplication::translate("MainWindowClass", "Add character stylesheets with formatting information.\n"
"You can select the stylesheets in your word processor to reformat additional text.", nullptr));
#endif // QT_CONFIG(tooltip)
        cbRTFCharStyles->setText(QCoreApplication::translate("MainWindowClass", "Add character styles", nullptr));
#if QT_CONFIG(tooltip)
        cbRTFPageColor->setToolTip(QCoreApplication::translate("MainWindowClass", "Set page color attribute to background color.", nullptr));
#endif // QT_CONFIG(tooltip)
        cbRTFPageColor->setText(QCoreApplication::translate("MainWindowClass", "Set page color", nullptr));
        label_9->setText(QCoreApplication::translate("MainWindowClass", "Page size:", nullptr));
        comboRTFPageSize->setItemText(0, QCoreApplication::translate("MainWindowClass", "A3", nullptr));
        comboRTFPageSize->setItemText(1, QCoreApplication::translate("MainWindowClass", "A4", nullptr));
        comboRTFPageSize->setItemText(2, QCoreApplication::translate("MainWindowClass", "A5", nullptr));
        comboRTFPageSize->setItemText(3, QCoreApplication::translate("MainWindowClass", "B4", nullptr));
        comboRTFPageSize->setItemText(4, QCoreApplication::translate("MainWindowClass", "B5", nullptr));
        comboRTFPageSize->setItemText(5, QCoreApplication::translate("MainWindowClass", "B6", nullptr));
        comboRTFPageSize->setItemText(6, QCoreApplication::translate("MainWindowClass", "Letter", nullptr));
        comboRTFPageSize->setItemText(7, QCoreApplication::translate("MainWindowClass", "Legal", nullptr));

#if QT_CONFIG(tooltip)
        comboRTFPageSize->setToolTip(QCoreApplication::translate("MainWindowClass", "Select a page size.", nullptr));
#endif // QT_CONFIG(tooltip)
        label_3->setText(QCoreApplication::translate("MainWindowClass", "I&mage width:", nullptr));
#if QT_CONFIG(tooltip)
        leSVGWidth->setToolTip(QCoreApplication::translate("MainWindowClass", "Enter the SVG width (may contain units).", nullptr));
#endif // QT_CONFIG(tooltip)
        label_12->setText(QCoreApplication::translate("MainWindowClass", "Height:", nullptr));
#if QT_CONFIG(tooltip)
        leSVGHeight->setToolTip(QCoreApplication::translate("MainWindowClass", "Enter the SVG height (may contain units).", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        cbSVGEmbedStyle->setToolTip(QCoreApplication::translate("MainWindowClass", "Include the style information in each output file.", nullptr));
#endif // QT_CONFIG(tooltip)
        cbSVGEmbedStyle->setText(QCoreApplication::translate("MainWindowClass", "Embed style (CSS)", nullptr));
        label_29->setText(QCoreApplication::translate("MainWindowClass", "Stylesheet file:", nullptr));
#if QT_CONFIG(tooltip)
        leSVGStyleFile->setToolTip(QCoreApplication::translate("MainWindowClass", "Name of the referenced style file.", nullptr));
#endif // QT_CONFIG(tooltip)
        leSVGStyleFile->setText(QCoreApplication::translate("MainWindowClass", "highlight.css", nullptr));
        label_30->setText(QCoreApplication::translate("MainWindowClass", "Stylesheet include file:", nullptr));
#if QT_CONFIG(tooltip)
        leSVGStyleIncFile->setToolTip(QCoreApplication::translate("MainWindowClass", "Path of the CSS include file.", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        pbSVGChooseStyleIncFile->setToolTip(QCoreApplication::translate("MainWindowClass", "Select a style include file.", nullptr));
#endif // QT_CONFIG(tooltip)
        pbSVGChooseStyleIncFile->setText(QCoreApplication::translate("MainWindowClass", "...", nullptr));
        label_7->setText(QCoreApplication::translate("MainWindowClass", "No options defined.", nullptr));
        tabWidgetOptions->setTabText(tabWidgetOptions->indexOf(page_output_spec), QCoreApplication::translate("MainWindowClass", "Output specific", nullptr));
        label_5->setText(QCoreApplication::translate("MainWindowClass", "Color theme:", nullptr));
#if QT_CONFIG(tooltip)
        comboTheme->setToolTip(QCoreApplication::translate("MainWindowClass", "Select a colour theme.", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        cbReformat->setToolTip(QCoreApplication::translate("MainWindowClass", "Reformat and indent your code.\n"
"This feature is enabled for C, C++, C# and Java code.", nullptr));
#endif // QT_CONFIG(tooltip)
        cbReformat->setText(QCoreApplication::translate("MainWindowClass", "Reformat:", nullptr));
#if QT_CONFIG(tooltip)
        comboReformat->setToolTip(QCoreApplication::translate("MainWindowClass", "Choose a formatting scheme.", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        cbKwCase->setToolTip(QCoreApplication::translate("MainWindowClass", "Change the keyword case.", nullptr));
#endif // QT_CONFIG(tooltip)
        cbKwCase->setText(QCoreApplication::translate("MainWindowClass", "Keyword case:", nullptr));
        comboKwCase->setItemText(0, QCoreApplication::translate("MainWindowClass", "UPPER", nullptr));
        comboKwCase->setItemText(1, QCoreApplication::translate("MainWindowClass", "lower", nullptr));
        comboKwCase->setItemText(2, QCoreApplication::translate("MainWindowClass", "Capitalize", nullptr));

#if QT_CONFIG(tooltip)
        comboKwCase->setToolTip(QCoreApplication::translate("MainWindowClass", "Select a keyword case.", nullptr));
#endif // QT_CONFIG(tooltip)
        label_6->setText(QCoreApplication::translate("MainWindowClass", "Tab width:", nullptr));
#if QT_CONFIG(tooltip)
        sbTabWidth->setToolTip(QCoreApplication::translate("MainWindowClass", "Enter the number of spaces which replace a tab.\n"
"Set the width to 0 to keep tabs.", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        cbWrapping->setToolTip(QCoreApplication::translate("MainWindowClass", "Enable line wrapping.", nullptr));
#endif // QT_CONFIG(tooltip)
        cbWrapping->setText(QCoreApplication::translate("MainWindowClass", "Line wrapping", nullptr));
#if QT_CONFIG(tooltip)
        sbLineLength->setToolTip(QCoreApplication::translate("MainWindowClass", "Enter the maximum line length.", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        cbAdvWrapping->setToolTip(QCoreApplication::translate("MainWindowClass", "Indent statements and function parameters after wrapping.", nullptr));
#endif // QT_CONFIG(tooltip)
        cbAdvWrapping->setText(QCoreApplication::translate("MainWindowClass", "Intelligent wrapping", nullptr));
#if QT_CONFIG(tooltip)
        cbOmitWrappedLineNumbers->setToolTip(QCoreApplication::translate("MainWindowClass", "Do not add line numbering to lines which were automatically wrapped.", nullptr));
#endif // QT_CONFIG(tooltip)
        cbOmitWrappedLineNumbers->setText(QCoreApplication::translate("MainWindowClass", "Omit line numbers of wrapped lines", nullptr));
        label_23->setText(QCoreApplication::translate("MainWindowClass", "Font na&me:", nullptr));
#if QT_CONFIG(tooltip)
        comboFontName->setToolTip(QCoreApplication::translate("MainWindowClass", "Select or enter the font name. HTML supports a list of fonts, separated with comma.", nullptr));
#endif // QT_CONFIG(tooltip)
        label_4->setText(QCoreApplication::translate("MainWindowClass", "Font si&ze:", nullptr));
#if QT_CONFIG(tooltip)
        leFontSize->setToolTip(QCoreApplication::translate("MainWindowClass", "Enter the font size.", nullptr));
#endif // QT_CONFIG(tooltip)
        leFontSize->setText(QCoreApplication::translate("MainWindowClass", "10", nullptr));
        tabWidgetOptions->setTabText(tabWidgetOptions->indexOf(page_formatting), QCoreApplication::translate("MainWindowClass", "Formatting", nullptr));
    } // retranslateUi

};

namespace Ui {
    class MainWindowClass: public Ui_MainWindowClass {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_MAINWINDOW_H
