/********************************************************************************
** Form generated from reading UI file 'showtextfile.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_SHOWTEXTFILE_H
#define UI_SHOWTEXTFILE_H

#include <QtCore/QVariant>
#include <QtGui/QIcon>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialog>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QTextBrowser>
#include <QtWidgets/QVBoxLayout>

QT_BEGIN_NAMESPACE

class Ui_ShowTextFile
{
public:
    QVBoxLayout *verticalLayout;
    QLabel *lbTitle;
    QTextBrowser *textBrowser;
    QHBoxLayout *horizontalLayout;
    QSpacerItem *horizontalSpacer;
    QPushButton *pushButton;

    void setupUi(QDialog *ShowTextFile)
    {
        if (ShowTextFile->objectName().isEmpty())
            ShowTextFile->setObjectName("ShowTextFile");
        ShowTextFile->setWindowModality(Qt::WindowModal);
        ShowTextFile->resize(711, 442);
        QIcon icon;
        icon.addFile(QString::fromUtf8(":/highlight.png"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        ShowTextFile->setWindowIcon(icon);
        ShowTextFile->setModal(true);
        verticalLayout = new QVBoxLayout(ShowTextFile);
        verticalLayout->setObjectName("verticalLayout");
        lbTitle = new QLabel(ShowTextFile);
        lbTitle->setObjectName("lbTitle");
        QFont font;
        font.setPointSize(10);
        font.setBold(true);
        lbTitle->setFont(font);

        verticalLayout->addWidget(lbTitle);

        textBrowser = new QTextBrowser(ShowTextFile);
        textBrowser->setObjectName("textBrowser");
        QFont font1;
        font1.setFamilies({QString::fromUtf8("Monospace")});
        textBrowser->setFont(font1);
        textBrowser->setOpenExternalLinks(true);

        verticalLayout->addWidget(textBrowser);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout->addItem(horizontalSpacer);

        pushButton = new QPushButton(ShowTextFile);
        pushButton->setObjectName("pushButton");

        horizontalLayout->addWidget(pushButton);


        verticalLayout->addLayout(horizontalLayout);


        retranslateUi(ShowTextFile);

        pushButton->setDefault(true);


        QMetaObject::connectSlotsByName(ShowTextFile);
    } // setupUi

    void retranslateUi(QDialog *ShowTextFile)
    {
        ShowTextFile->setWindowTitle(QCoreApplication::translate("ShowTextFile", "Show text", nullptr));
        lbTitle->setText(QCoreApplication::translate("ShowTextFile", "TextLabel", nullptr));
        pushButton->setText(QCoreApplication::translate("ShowTextFile", "OK", nullptr));
    } // retranslateUi

};

namespace Ui {
    class ShowTextFile: public Ui_ShowTextFile {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_SHOWTEXTFILE_H
